#include <q3listview.h>
#include <qobject.h>
#include <qmessagebox.h>
#include <qinputdialog.h>

#include <kglobal.h>

#include <kgenericfactory.h>
#include <kdebug.h>
#include <kconfig.h>
#include <kapplication.h>
#include "../constants.h"
#include "interactive.h"

K_EXPORT_COMPONENT_FACTORY( kupdateapplet_interactive, KGenericFactory<InteractivePlugin>( "kupdateapplet_interactive" ) )

InteractivePlugin::InteractivePlugin( QObject *parent, const QStringList& ) 
    : Updater( parent )
    , _patchList(0)
    , _packageList(0)

{
}

// this defines what this plugin is able to provide
UpdaterCapabilities InteractivePlugin::capabilities()
{
	UpdaterCapabilities caps;
	caps.canSelectIndividualUpdates = true;
	caps.canProvidePackages = true;
	return caps;
}


void InteractivePlugin::shutdown()
{
	QMessageBox::information(NULL, "shutdown()", "Interactive Plugin should shutdown.");
}


// the applet wants us to check for updates and add them them to patchlist and packagelist
void InteractivePlugin::slotPopulateLists(Q3ListView *patchList, Q3ListView *packageList )
{
	_patchList = patchList;
	_packageList = packageList;

	// in real life we'd need to clear the lists
	// but in this example we want the list to grow from check to check


	// add updates to the list
	
	QMessageBox::information(NULL, 
				"slotPopulateLists()", 
				"Please provide a list of Updates...");

	if (QMessageBox::question(NULL,
				"slotPopulateLists()",
				"Add a security patch?",
				QMessageBox::Yes | QMessageBox::No ) == QMessageBox::Yes )
	{
		Q3CheckListItem *newItem;
		newItem = new Q3CheckListItem ( _patchList, "somepatch", Q3CheckListItem::CheckBox );
		newItem->setText(COLUMN_NAME, "somepatch" );
		newItem->setText(COLUMN_SUMMARY, "this is an example" );
		newItem->setText(COLUMN_TYPE, "security" );
		newItem->setText(COLUMN_NEW_VERSION, "0.1" );
		newItem->setText(COLUMN_CATALOG, "somerepo" );
		newItem->setText(COLUMN_ID, "somepatch-01" ) ;
	}

	if (QMessageBox::question(NULL,
				"slotPopulateLists()",
				"Add a recommended patch?",
				QMessageBox::Yes | QMessageBox::No ) == QMessageBox::Yes )
	{
		Q3CheckListItem *newItem;
		newItem = new Q3CheckListItem ( _patchList, "somepatch1", Q3CheckListItem::CheckBox );
		newItem->setText(COLUMN_NAME, "somepatch1" );
		newItem->setText(COLUMN_SUMMARY, "this is an example" );
		newItem->setText(COLUMN_TYPE, "recommended" );
		newItem->setText(COLUMN_NEW_VERSION, "0.2" );
		newItem->setText(COLUMN_CATALOG, "somerepo" );
		newItem->setText(COLUMN_ID, "somepatch-02" ) ;
	}

	if (QMessageBox::question(NULL,
				"slotPopulateLists()",
				"Add an optional  patch?",
				QMessageBox::Yes | QMessageBox::No ) == QMessageBox::Yes )
	{
		Q3CheckListItem *newItem;
		newItem = new Q3CheckListItem ( _patchList, "somepatch3", Q3CheckListItem::CheckBox );
		newItem->setText(COLUMN_NAME, "somepatch3" );
		newItem->setText(COLUMN_SUMMARY, "this is an example" );
		newItem->setText(COLUMN_TYPE, "optional" );
		newItem->setText(COLUMN_NEW_VERSION, "0.3" );
		newItem->setText(COLUMN_CATALOG, "somerepo" );
		newItem->setText(COLUMN_ID, "somepatch-03" ) ;
	}

	if (QMessageBox::question(NULL,
				"slotPopulateLists()",
				"Add a package?",
				QMessageBox::Yes | QMessageBox::No ) == QMessageBox::Yes )
	{
		Q3CheckListItem *newItem;
		newItem = new Q3CheckListItem ( _packageList, "somepackage1", Q3CheckListItem::CheckBox );
		newItem->setText(COLUMN_NAME, "somepackage1" );
		newItem->setText(COLUMN_SUMMARY, "this is an example" );
		newItem->setText(COLUMN_NEW_VERSION, "0.4" );
		newItem->setText(COLUMN_CATALOG, "somerepo" );
		newItem->setText(COLUMN_ID, "somepackage1-04" ) ;
	}


	// do some progress information

	for (int i=0; i <= 100; i+=20)
	{
		if (QMessageBox::question(NULL, 
				"slotPopulateLists()",
				QString("Ok = set progress to %1\%\nCancel = exit with an error").arg(i),
				QMessageBox::Ok| QMessageBox::Cancel ) == QMessageBox::Ok )
		{
			emit progress(i, 100, "progress while checking for updates");
		}
		else		
		{
			emit appletError( "Check failed. User clicked cancel" );
			return;
		}
	}

	emit checkFinished();
}

// the applet wants us to send a update description
void InteractivePlugin::slotResolvableSelected(Q3ListViewItem *item, int resolvableType)
{
	QString description = QInputDialog::getText(NULL,"resolvableselected()","An update has been selected. Please provide a description");
	emit returnDescription(description, resolvableType);
}

// the applet wants us to start the installation
void InteractivePlugin::slotStartInstall()
{
	if (!_patchList || !_packageList)
		return;

	QMessageBox::information(NULL,"slotStartInstall()","Let's start the installation.");

	for (int i=0; i <= 100; i+=20)
	{
		if (QMessageBox::question(NULL, 
										"slotStartInstall()",
										QString("Ok = set progress to %1\%\nCancel = exit with an error").arg(i),
										QMessageBox::Ok| QMessageBox::Cancel ) == QMessageBox::Ok )
		{
			emit progress(i, 100, "progress while installing");
		}
		else		
		{
			emit appletError( "Install failed. User clicked cancel" );
			return;
		}
	}

	emit checkFinished();


}



// the applet wants us to show our configuration dialog
void InteractivePlugin::slotConfigureUpdater()
{
	QMessageBox::information(NULL,"slotConfigureUpdater()","Configuration of our plugin has been requested.");
}





#include "interactive.moc"

// vim: ts=2 noet sw=2
