#include <QString>
#include <QObject>
#include <QTreeWidget>
#include <QDir>
#include <QTimer>
#include <QProcess>
#include <klocale.h>
#include <solid/networking.h>
#include <solid/devicenotifier.h>

#include "kconfigdialog.h"
#include "configwidget.h"
#include "kupdateapplet_settings.h"
#include "constants.h"
#include "updater.h"
#include "logics.h"
#include "distroupgrades.h"


QDebug Logics::debugStateOn( QDebug str ) const
{
    switch ( _state )
    {
        case  APPLET_NO_UPDATES: str << ". state: APPLET_NO_UPDATES"; break;
        case  APPLET_UPDATES : str << ". state: APPLET_UPDATE"; break;
        case  APPLET_CRITICAL_UPDATES: str << ". state: APPLET_CRITICAL_UPDATES"; break;
        case  APPLET_CHECKING: str << ". state: APPLET_CHECKING"; break;
        case  APPLET_PROBLEM: str << ". state: APPLET_PROBLEM"; break;
        case  APPLET_INSTALLING: str << ". state: APPLET_INSTALLING"; break;
        case  APPLET_PKGMGR_UPDATES: str << ". state: APPLET_PKGMGR_UPDATES"; break;
        case  APPLET_NO_UPDATE_REPO: str << ". state: APPLET_NO_UPDATE_REPO"; break;
        case  APPLET_ONLY_OPTIONAL_UPDATES: str << ". state: APPLET_ONLY_OPTIONAL_UPDATES"; break;
        case  APPLET_NO_NETWORK: str << ". state: APPLET_NO_NETWORK"; break;
        case  APPLET_SYSTEM_BUSY_WHILE_CHECK: str << ". state: APPLET_SYSTEM_BUSY_WHILE_CHECK"; break;
        case  APPLET_SYSTEM_BUSY_WHILE_INSTALL: str << ". state: APPLET_SYSTEM_BUSY_WHILE_INSTALL"; break;
        case  APPLET_SYSTEM_IS_ON_BATTERY_WHILE_CHECK: str << ". state: APPLET_SYSTEM_IS_ON_BATTERY_WHILE_CHECK"; break;
        case  APPLET_SYSTEM_IS_ON_BATTERY_WHILE_INSTALL: str << ". state: APPLET_SYSTEM_IS_ON_BATTERY_WHILE_INSTALL"; break;
        case  APPLET_PLUGIN_PROBLEM: str << ". state: APPLET_PLUGIN_PROBLEM"; break;
        case  APPLET_NEW_HARDWARE: str << ". state: APPLET_NEW_HARDWARE"; break;
        default: str << "! state: unknown state"; break;
    }
    str << ". state description: " << _stateDescription ;

    return str;
    
}

void Logics::setState( int state, const QString& desc )
{
	_state = state;
	_stateDescription = desc;

// FIXME doesn't work in autobuild
//        debugStateOn(kDebug());

	dumpUpdateLists();

	emit stateChanged();


}

void Logics::selectAllPatches()
{
	if (!_patchList)
		return;

        QTreeWidgetItemIterator it(_patchList);
        while (*it)
        {
            QTreeWidgetItem *item(*it);
            if (item->checkState(0) != Qt::Checked)
                item->setCheckState(0, Qt::Checked);
            ++it;
        }

	kDebug() << "$ all patches selected";
	emit selectionChanged();
}


void Logics::selectAllPackages()
{
	if (!_packageList)
		return;

        QTreeWidgetItemIterator it(_packageList);
        while (*it)
        {
            QTreeWidgetItem *item(*it);
            if (item->checkState(0) != Qt::Checked)
                item->setCheckState(0, Qt::Checked);
            ++it;
        }

	kDebug() << "$ all package selected";
	emit selectionChanged();
}

void Logics::selectAllDrivers()
{
	if (!_driverList)
		return;

        QTreeWidgetItemIterator it(_driverList);
        while (*it)
        {
            QTreeWidgetItem *item(*it);
            if (!item->checkState(0) == Qt::Checked)
                item->setCheckState(0, Qt::Checked);
            ++it;
        }
        
	kDebug() << "$ all drivers selected";
	emit selectionChanged();
}

QStringList Logics::driversStringList()
{
	QStringList drivers;

	if (!_driverList)
		return drivers;

        QTreeWidgetItemIterator it(_driverList);
        while (*it)
        {
                QTreeWidgetItem *item(*it);
		drivers << item->text(COLUMN_NAME);
		++it;
	}

	return drivers;
}

void Logics::deselectAllPatches()
{
	if (!_patchList)
		return;

        QTreeWidgetItemIterator it(_patchList);
        while (*it)
        {
            QTreeWidgetItem *item(*it);
            if (item->checkState(0) == Qt::Checked)
            {    
                item->setCheckState(0, Qt::Unchecked);
                item->setSelected(false);
            }
            
            ++it;
        }

	kDebug() << "$ all patches deselected";
	emit selectionChanged();
}


void Logics::deselectAllPackages()
{
	if (!_packageList)
		return;

        QTreeWidgetItemIterator it(_packageList);
        while (*it)
        {
            QTreeWidgetItem *item(*it);
            if (item->checkState(0) == Qt::Checked)
            {    
                item->setCheckState(0, Qt::Unchecked);
                item->setSelected(false);
            }
            
            ++it;
        }

	kDebug() << "$ all packages deselected";
	emit selectionChanged();
}


void Logics::selectRecommendedPatches()
{
	if (!_patchList)
		return;

	if ( ! (_caps & Updater::CanSelectIndividualUpdates) )
	        return;

        QTreeWidgetItemIterator it(_patchList);
        while (*it)
        {
            QTreeWidgetItem *item(*it);
        	if (item->text(COLUMN_TYPE).toLower() == "recommended"  ||
                    item->text(COLUMN_TYPE).toLower() == "packagemanager"  )
            {    
                item->setCheckState(0, Qt::Checked);
            }
            
            ++it;
        }

	kDebug() << "$ recommended patches selected";
	emit selectionChanged();
}


void Logics::selectStandardPatches()
{
	if (!_patchList)
		return;

	if ( ! (_caps & Updater::CanSelectIndividualUpdates) )
		return;

        QTreeWidgetItemIterator it(_patchList);
        while (*it)
        {
            QTreeWidgetItem *item(*it);
        	if (item->text(COLUMN_TYPE).toLower() == "security"  ||
                    item->text(COLUMN_TYPE).toLower() == "packagemanager"  )
                    item->setCheckState(0, Qt::Checked);
                else
                    item->setCheckState(0, Qt::Unchecked);
               
            ++it;
        }

	kDebug() << "$ standard patches selected";
	emit selectionChanged();
}


void Logics::deselectRestartPatches()
{
	if (!_patchList)
		return;

	if ( ! ( _caps & Updater::CanSelectIndividualUpdates ) )
		return;

        QTreeWidgetItemIterator it(_patchList);
        while (*it)
        {
            QTreeWidgetItem *item(*it);
            if (item->text(COLUMN_RESTART_HIDDEN).toLower() == "system"  ||
                item->text(COLUMN_TYPE).toLower() == "packagemanager"  )
                item->setCheckState(COLUMN_CHECK, Qt::Unchecked);
               
            ++it;
        }

	kDebug() << "$ restart patches deselected";
	emit selectionChanged();
}


int Logics::selectedPackages()
{
	if (!_packageList)
		return 0;

	int packagesSelected=0;

        QTreeWidgetItemIterator it(_packageList);
        while (*it)
        {
            QTreeWidgetItem *item(*it);
            if ( item->checkState(COLUMN_CHECK) == Qt::Checked )
                packagesSelected++;               
            ++it;
        }

	kDebug() << "# selected packages: " << packagesSelected;
	return packagesSelected;
}


int Logics::selectedPatches()
{
	if (!_patchList)
		return 0; 

	int patchesSelected=0;

        QTreeWidgetItemIterator it(_patchList);
        while (*it)
        {
            QTreeWidgetItem *item(*it);
            if ( item->checkState(COLUMN_CHECK) == Qt::Checked )
                patchesSelected++;               
            ++it;
        }

	kDebug() << "# selected patches: " << patchesSelected;
	return patchesSelected;
}


int Logics::recommendedPatches()
{
	if (!_patchList)
		return 0;

	int count=0;

        QTreeWidgetItemIterator it(_patchList);
        while (*it)
        {
            QTreeWidgetItem *item(*it);
            if ((item->text(COLUMN_TYPE).toLower() == "recommended" ||
                 item->text(COLUMN_TYPE).toLower() == "packagemanager"  ))
                count++;
            ++it;
        }

	kDebug() << "# recommended patches: " << count;
	return count;
}

int Logics::drivers()
{
	if (!_driverList)
		return 0;

	int count = _driverList->topLevelItemCount();

	kDebug() << "# drivers: " << count;
	return count;
}



int Logics::selectedRecommendedPatches()
{
	if (!_patchList)
		return 0;

	int count=0;

        QTreeWidgetItemIterator it(_patchList);
        while (*it)
        {
            QTreeWidgetItem *item(*it);
            if ((item->text(COLUMN_TYPE).toLower() == "recommended" ||
                 item->text(COLUMN_TYPE).toLower() == "packagemanager"  ) &&
                (item->checkState(COLUMN_CHECK) == Qt::Checked))
                count++;

            ++it;
        }

	kDebug() << "# selected recommended patches: " << count;
	return count;
}


int Logics::securityPatches()
{
	if (!_patchList)
		return 0;

	int count=0;

        QTreeWidgetItemIterator it(_patchList);
        while (*it)
        {
            QTreeWidgetItem *item(*it);
		if (item->text(COLUMN_TYPE).toLower() == "security" )
                    count++;
            ++it;
        }

	kDebug() << "# security patches: " << count;
	return count;
}


int Logics::packagemanagerPatches()
{	
	if (!_patchList)
		return 0;

	int count=0;
        QTreeWidgetItemIterator it(_patchList);
        while (*it)
        {
            QTreeWidgetItem *item(*it);
            if (item->text(COLUMN_TYPE).toLower() == "packagemanager" )
                count++;
            ++it;
        }
	kDebug() << "# packagemanager patches: " << count;
	return count;
}


int Logics::selectedSecurityPatches()
{	
	if (!_patchList)
		return 0;

	int count=0;

        QTreeWidgetItemIterator it(_patchList);
        while (*it)
        {
            QTreeWidgetItem *item(*it);
            
            if (item->text(COLUMN_TYPE).toLower() == "security" &&
                (item->checkState(COLUMN_CHECK) == Qt::Checked))
                count++;
            ++it;
        }

	kDebug() << "# selected security patches: " << count;
	return count;
}


int Logics::restartPatches()
{
	if (!_patchList)
		return 0;

	int count=0;

        QTreeWidgetItemIterator it(_patchList);
        while (*it)
        {
            QTreeWidgetItem *item(*it);
            if (item->text(COLUMN_RESTART_HIDDEN).toLower() == "system")
                count++;
            ++it;
        }

	kDebug() << "# restart patches: " << count;
	return count;
}


int Logics::selectedRestartPatches()
{
	if (!_patchList)
		return 0;

	int count=0;

        QTreeWidgetItemIterator it(_patchList);
        while (*it)
        {
            QTreeWidgetItem *item(*it);
            if (item->text(COLUMN_RESTART_HIDDEN).toLower() == "system" &&
                (item->checkState(COLUMN_CHECK) == Qt::Checked))
                count++;
            ++it;
        }

	kDebug() << "# selected restart patches: " << count;
	return count;
}


int Logics::packages()
{
	if (!_packageList)
            return 0;

	int count = _packageList->topLevelItemCount();


	kDebug() << "# packages: " << count;
        
	return count;
}


int Logics::patches()
{	
	if (!_patchList)
            return 0;

	int count = _patchList->topLevelItemCount();

	kDebug() << "# patch: " << count;
        
	return count;

}



bool Logics::systemIsBusy()
{
	double loadavg, maxloadavg;

	getloadavg( &loadavg, 1 );  //get loadavg over last minute
	maxloadavg = UpdaterSettings::self()->maxSystemLoad();

	kDebug() <<  "+ max system load: " << maxloadavg;
	kDebug() <<  "+ system load: " << loadavg;

	if (loadavg > maxloadavg )
	{
		kDebug() << "? system is busy"; 
		return true;
	}
	else
	{
		kDebug() << ". system is not busy"; 
		return false;
	}
}


bool Logics::systemIsOnBattery()
{
	if (system( "/usr/bin/on_ac_power" ) > 0)
	{
		kDebug() << "? system is on battery"; 
		return true;
	}
	else
	{
		kDebug() << ". system is not on battery"; 
		return false;
	}
}

void Logics::slotProgress(int progress, int totalSteps, const QString &desc )
{
	_progress = progress;	
	_progressTotal = totalSteps;
	_progressDescription = desc;
	emit progressChanged();
}



void Logics::slotPluginError( const QString &desc )
{
	setState(APPLET_PLUGIN_PROBLEM, desc);
	kWarning() << "! plugin error: " << desc;
}

void Logics::slotAppletError( const UpdaterError &error )
{
	// when the state was 'installing' enable autosuspend again
	if (_state == APPLET_INSTALLING )
		disableAutoSuspend( false );

	setState ( APPLET_PROBLEM, error.errorMessage() );
	kWarning() << "! applet error: " << error.errorMessage();
}

void Logics::slotAppletError( const QString &desc )
{
	// when the state was 'installing' enable autosuspend again
	if (_state == APPLET_INSTALLING )
		disableAutoSuspend( false );

	setState ( APPLET_PROBLEM, desc );
	kWarning() << "! applet error: " << desc;
}

void Logics::slotAppletMessage( const QString &desc )
{
	// we collect messages that arrive within a short time frame
	// and show them as a single notification 	

	kDebug() << ". new message stored";
	kDebug() << ". message description: " << desc;
	_messages << desc;
	QTimer::singleShot(2*1000, this, SLOT(slotMessageCollection()));
}

void Logics::slotMessageCollection ()
{
	if (_messages.empty())
		return;

	kDebug() << ". display messages";
	emit message(_messages.join("\n"));
	_messages.clear();
}



void Logics::slotPluginNeedsConfiguration ( int cause )
{
	// when the state was 'installing' enable autosuspend again
	if (_state == APPLET_INSTALLING )
		disableAutoSuspend( false );

	if (cause == PLUGIN_CONFIG_NO_UPDATE_REPOS )
	{
		setState ( APPLET_NO_UPDATE_REPO );
		kWarning() << "? plugin needs configuration: no update repos";
	}
	else
	{
		// this part is reserved for new scenarios 
		// at the moment it behaves like above
		setState ( APPLET_NO_UPDATE_REPO );
		kWarning() << "? plugin needs configuration: unknown cause";
	}
}

void Logics::slotInstallFinished()
{
	kDebug() << ". install finished";

	disableAutoSuspend( false );
	slotAutoCheckForUpdates();
}


void Logics::writeAutostartFile()
{
	// -----------------------------------------------------------
	// check whether auto start configuration has changed
	// create or delete autostart file according to configuration

	if ( !UpdaterSettings::self()->autostart() )
	{
		QDir configDir( QDir::home().absolutePath() + "/.config" );
		if ( !configDir.exists() )
			configDir.mkdir( QDir::home().absolutePath() + "/.config" );

		QDir configAutostartDir( QDir::home().absolutePath() + "/.config/autostart" );
		if ( !configAutostartDir.exists() )
			configAutostartDir.mkdir( QDir::home().absolutePath() + "/.config/autostart" );

		QFile file( QDir::home().absolutePath() + "/.config/autostart/kupdateapplet-autostart.desktop" );
		if ( !file.open( QIODevice::WriteOnly | QIODevice::Text ) )
			return;

		QTextStream out( &file );
		out << "[Desktop Entry]" << endl;
		out << "Hidden=true" << endl;
	}
	else
	{
		QFile file( QDir::home().absolutePath() + "/.config/autostart/kupdateapplet-autostart.desktop" );
		if ( file.exists() )
			file.remove();
	}
}


void Logics::slotConfigChanged()
{
	kDebug() << ". config changed";
	writeAutostartFile();

	int  timerInterval = UpdaterSettings::self()->interval() * 60 * 60 * 1000; // convert to ms
	_checkTimer.setInterval( timerInterval );

	emit configurationChanged();
}


void Logics::slotCheckFinished()
{
	kDebug() << ". check finished";

	if ( drivers() > 0 )
	{
		if (!_lastHardware.isEmpty())
                    setState( APPLET_NEW_HARDWARE, _lastHardware.first().product() );
		else
			setState( APPLET_NEW_HARDWARE, driversStringList().join("\"<br>\"") );

	}
	else if ( packagemanagerPatches() > 0 )
		setState( APPLET_PKGMGR_UPDATES );
	else if ( securityPatches() > 0 )
		setState( APPLET_CRITICAL_UPDATES );
	else if ( recommendedPatches() > 0 )
	  	setState( APPLET_UPDATES );
	else if ( patches() > 0 )
		setState( APPLET_ONLY_OPTIONAL_UPDATES );
	else if ( (updaterCapabilities() & Updater::CanProvidePackages  )
		&& UpdaterSettings::self()->showPackages()
 		&& packages() > 0 ) 
		setState (APPLET_ONLY_OPTIONAL_UPDATES);
	else
		setState ( APPLET_NO_UPDATES );

	// set last check time
	UpdaterSettings::self()->setState(_state);
	UpdaterSettings::self()->setLastCheckTime(QDateTime::currentDateTime() );
	UpdaterSettings::self()->writeConfig();

	// check whether a distro upgrade is available
	checkForDistroUpgrade(); 

	// show "send system profile" popup after 5 minutes if needed
	QTimer::singleShot(5*60*1000, this, SLOT(slotSystemProfilePopup()));
}


void Logics::configurePlugin()
{
	kDebug() << ". configure plugin";
	emit configure();
}


void Logics::getDescription( QTreeWidgetItem *resolvable, int type )
{
	kDebug() << ". get description";
	emit getResolvableDescription( resolvable, type);
}


void Logics::slotResolvableDescription( const QString & desc, int type )
{
	kDebug() << ". got description: " << desc;
	emit resolvableDescription( desc, type);
}

void Logics::installDrivers()
{
	kDebug() << ". start install drivers";


	deselectAllPatches();
	deselectAllPackages();
	selectAllDrivers();

	setState( APPLET_INSTALLING );

	disableAutoSuspend( true );
	emit install();
}


void Logics::startInstall( bool force )
{
	kDebug() << ". start install";

	setState( APPLET_INSTALLING );
	
	if ( force )
	{
		kDebug() << "! this is a forced installation";
	}
	else if ( !systemReady() )
	{
		kDebug() << "! system is not ready for installation";
		return;
	}

	disableAutoSuspend( true );
	emit install();
}

void Logics::startManualInstallation()
{

	emit stopPackageManagementDaemon();

	void slotStopPackageManagementDeamon();
	kDebug() << ". start manual installation";

        QProcess* helperprocess = new QProcess(this);
        connect( helperprocess, SIGNAL(finished(int, QProcess::ExitStatus)), SLOT(slotHelperFinished(int, QProcess::ExitStatus)));
        helperprocess->start("kdesu yast2 online_update");
}

void Logics::slotHelperFinished(int exitCode, QProcess::ExitStatus exitStatus)
{
	kDebug() << ". helper process finished:";
	kDebug() << "  helper exit code: " << exitCode;
	kDebug() << "  helper exit status: " << exitStatus;

	slotAutoCheckForUpdates();
}


void Logics::startConfigureUpdateRepo()
{
	kDebug() << ". start 'configure update repo' workflow";

        QProcess* helperprocess = new QProcess(this);
        connect( helperprocess, SIGNAL(finished(int, QProcess::ExitStatus)), SLOT(slotHelperFinished(int, QProcess::ExitStatus)));
        helperprocess->start("kdesu yast2 online_update_configuration");

}


Logics::Logics()
	: _state( APPLET_NO_UPDATES )
	, _debugLogging( true )
	, _stateDescription ("")
	, _progress ( 0 )
	, _progressTotal (100)
	, _progressDescription ("")
	, _skipAutoCheck( false )
	, _systemProfilePopupShownThisSession ( false )
{
	kDebug() << ": loading logics";

	// initialize timer for regularly checking
	int timerInterval = UpdaterSettings::self()->interval() * 60 * 60 * 1000; // convert to ms
	_checkTimer.setInterval( timerInterval );
	connect( &_checkTimer, SIGNAL( timeout() ), 
		 this,	 SLOT( slotAutoCheckForUpdates() ) );


	// calculate timer for first check
	QDateTime lastCheckTime = UpdaterSettings::self()->lastCheckTime();
	QDateTime now = QDateTime::currentDateTime();
	int interval = UpdaterSettings::self()->interval();
	int nextCheck = ( interval * 60 * 60 ) - lastCheckTime.secsTo( now );

	if ( nextCheck < 30 )
		nextCheck = 30;

	kDebug() <<  ": last check: " << lastCheckTime.toString();
	kDebug() <<  ": now: %1" << now.toString();
	kDebug() <<  ": interval [s]: %1" << interval * 60 * 60;
	kDebug() <<  ": next check in [s]: %1" << nextCheck;

	QTimer::singleShot( nextCheck * 1000, this , SLOT( slotAutoCheckForUpdates() ) );


	// connect to device notifier that informs us about
	// newly attachted devices that need drivers installed
        // so we can recommend drivers
        connect( Solid::DeviceNotifier::instance(),
                SIGNAL(deviceAdded(const QString &)),
                SLOT(slotDeviceAdded(const QString &)) );



	QTimer::singleShot( 3 * 1000, this , SLOT( slotCheckFirmware() ) );
}


void Logics::checkForUpdates( bool force )
{
	if (!_patchList || !_packageList || !_driverList)
        {
            kDebug() << "can't check for updates";
            return;
        }
        
	kDebug() << ". user triggered check for updates";

	setState ( APPLET_CHECKING );

	if ( force )
	{
		kDebug() << "! this is a forced check";
	}
	else if ( !systemReady() )	// systemReady() sets the state
	{
		kDebug() << "! system is not ready for check";
		return;
	}

	_skipAutoCheck = true;
	emit stateChanged();
	emit checkUpdates(_patchList, _packageList, _driverList);
}


void Logics::slotAutoCheckForUpdates()

{	if (!_patchList || !_packageList || !_driverList)
		return;

	kDebug() << ". automatically triggerd check for updates";

	setState ( APPLET_CHECKING );

	if ( !systemReady() )	// systemReady() sets the state 
	{
		kDebug() << "! system is not ready for check";
		return;
	}

	emit checkUpdates(_patchList, _packageList, _driverList);

}

void Logics::slotAutoCheckForUpdatesCauseNotReady()
{

	if ( _skipAutoCheck )
	{
		kDebug() << ". skip automatically triggerd check for updates because system was not ready";
	}
	else
	{
		kDebug() << ". automatically triggerd check for updates because system was not ready";
		slotAutoCheckForUpdates();
	}
}



bool Logics::systemReady()
{
	// checks whether the system is idle, connected to power supply and network 
	kDebug() << ". checking wheter system is ready";

	// check whether system is idle or busy
	if ( UpdaterSettings::self()->skipCheckSystemBusy() &&  systemIsBusy() )
	{
		kDebug() << "! system is busy";
		if ( _state == APPLET_CHECKING )
		{
			setState( APPLET_SYSTEM_BUSY_WHILE_CHECK );
			_skipAutoCheck = false;
			QTimer::singleShot( 1000 * 60, this , SLOT( slotAutoCheckForUpdatesCauseNotReady() ) );
		}
		else
		{
			setState( APPLET_SYSTEM_BUSY_WHILE_INSTALL );
		}
            
        	return false;
	}
	else
	{
		kDebug() << ". system is idle";
	}


	// check whether system is on battery
	if ( UpdaterSettings::self()->skipCheckOnBattery() && systemIsOnBattery() )
	{
		kDebug() << "! system is on battery";
		if ( _state == APPLET_CHECKING )
		{
			setState( APPLET_SYSTEM_IS_ON_BATTERY_WHILE_CHECK );
			_skipAutoCheck = false;
			QTimer::singleShot( 1000 * 30, this , SLOT( slotAutoCheckForUpdatesCauseNotReady() ) );
		}
		else
		{
			setState( APPLET_SYSTEM_IS_ON_BATTERY_WHILE_INSTALL );
		}
        
		return false;
	}
	else
		kDebug() << ". system is connected to power";



	if ( Solid::Networking::status() != Solid::Networking::Connected  &&
		Solid::Networking::status() != Solid::Networking::Unknown )
	{
		kDebug() << "! system is not connect to the network";
		if ( _state == APPLET_CHECKING )
		{
			setState( APPLET_NO_NETWORK );
			_skipAutoCheck = false;
			QTimer::singleShot( 1000 * 30, this , SLOT( slotAutoCheckForUpdatesCauseNotReady() ) );
			return false;
		}
		else
		{
			// setState( APPLET_NO_NETWORK );
			// return false;
			// FIXME and TODO
			kDebug() << ". don't complain about missing network";
			kDebug() << ". avoid blocking state because of missing network ";
			kDebug() << ". assume network is avaiablable ";
		}
	}
	else
		kDebug() << ". system is connected to network";

	return true;
}



Logics::~Logics()
{
	kDebug() << "~Logics()" << endl;
}


void Logics::configureApplet()
{
	kDebug() << ". configure applet";
	static ConfigWidget* confWidget = 0;

	if ( KConfigDialog::showDialog( "settings" ) )
		return;

	_configDialog = new KConfigDialog( NULL, "settings", UpdaterSettings::self() );
	_configDialog->setFaceType( KPageDialog::Plain );
	_configDialog->setHelp( QString::null );
	_configDialog->setButtons( KDialog::Ok | KDialog::Cancel );
	confWidget = new ConfigWidget( 0 );
	_configDialog->addPage( confWidget, PROGRAM_DISPLAY_NAME, PROGRAM_ICON, i18n( "Settings" ) );
	connect( _configDialog, SIGNAL( settingsChanged( const QString& ) ), SLOT( slotConfigChanged() ) );
	_configDialog->show();
}

void Logics::disableAutoSuspend(bool value)
{
	kDebug() << ". disableAutoSuspend set to: " << value;

	// TODO
	// needs to wait until kpowersave is able to listen to dbus
	kDebug() << "! disableAutoSuspend not implemented yet";
}



void Logics::dumpUpdateLists()
{	

    if (!_patchList || !_packageList || !_driverList)
    {
        kDebug() << "list not propertly initialized";
        return;
    }
    
	kDebug() << ". ----- package, patch and driver list --------------------";
        // iterate
        {    
            QTreeWidgetItemIterator it(_patchList);
            while (*it)
                    {
                        QTreeWidgetItem *item(*it);
                        if (item->checkState(COLUMN_CHECK) == Qt::Checked)
                            kDebug() << ". [x] patch: " << item->text(COLUMN_NAME);
			else
                            kDebug() << ". [ ] patch: " << item->text(COLUMN_NAME);
                        ++it;
                    }
        }
        
        // iterate
        {    
                    QTreeWidgetItemIterator it(_packageList);
                    while (*it)
                    {
                        QTreeWidgetItem *item(*it);
                        if (item->checkState(COLUMN_CHECK) == Qt::Checked)
                            kDebug() << ". [x] package: " << item->text(COLUMN_NAME);
			else
                            kDebug() << ". [ ] package: " << item->text(COLUMN_NAME);
                        ++it;
                    }
        }

        // iterate
        {    
            QTreeWidgetItemIterator it(_driverList);
            while (*it)
            {
                QTreeWidgetItem *item(*it);
                if (item->checkState(COLUMN_CHECK) == Qt::Checked)
                    kDebug() << ". [x] driver: " << item->text(COLUMN_NAME);
                else
                    kDebug() << ". [ ] driver: " << item->text(COLUMN_NAME);
                ++it;
            }
        }
        
	kDebug() << ". ----- end of package, patch and driver list -------------";
}


void Logics::slotDeviceAdded( const QString &udi )
{
	kDebug() << "@ new hardware has been attached";
	kDebug() << "@ udi: " << udi;
	Solid::Device device(udi);


	if ( !device.isValid() )
	{
		kDebug() << "! device is not valid";
		return;
	}
	
	kDebug() << "@ device is valid";
	kDebug() << "@ udi: " << device.udi();
	kDebug() << "@ parentUid: " << device.parentUdi();
	kDebug() << "@ product: " << device.product();
	kDebug() << "@ vendor: " << device.vendor();
	kDebug() << "@ icon: " << device.icon();

	if ( ! UpdaterSettings::self()->driverSearch() )
	{
		kDebug() << ". driver search is disabled";
		return;
	}

	_lastHardware << device;

        if (!_patchList || !_packageList || !_driverList)
        {
            kDebug() << "list not propertly initialized";
            return;
        }

	kDebug() << ". searching for drivers";
	setState ( APPLET_CHECKING );

	emit stateChanged();
	emit checkDrivers(_patchList, _packageList, _driverList);

	QTimer::singleShot( 5 * 60 * 1000, this , SLOT( slotClearHardwareList() ) );
}

void Logics::slotCheckFirmware()
{
	if (!_driverList || !_packageList || !_patchList )
		return;

	emit checkFirmware(_patchList, _packageList, _driverList);
}

void Logics::slotClearHardwareList()
{
	// clears the list of attached hardware
	// if a more devices are attached at the same time
	// we want to show all of them to the user
	// after some time this list we want to clear this
	// list as we don't know whether the user has configured 
	// has installed the drivers already
	_lastHardware.clear();
}


void Logics::addDummyPatch(int state)
{
	QTreeWidgetItem* newItem = new QTreeWidgetItem (_patchList);
	newItem->setFlags(Qt::ItemIsUserCheckable
		| Qt::ItemIsEnabled
		| Qt::ItemIsSelectable );
        
	newItem->setText(COLUMN_NAME, "dummy patch for applet state" );

        if (state == APPLET_CRITICAL_UPDATES )
		newItem->setText(COLUMN_TYPE, "security" );
	else if (state == APPLET_PKGMGR_UPDATES )
		newItem->setText(COLUMN_TYPE, "packagemanager" );
}


void Logics::checkForDistroUpgrade()
{
	QDateTime lastDistroUpgradeCheckTime = UpdaterSettings::self()->lastDistroUpgradeCheckTime();
	QDateTime now = QDateTime::currentDateTime();

	kDebug() <<  ": last checkDistroUpgrade: " << lastDistroUpgradeCheckTime.toString();

	if ( lastDistroUpgradeCheckTime.addDays( DISTROUPGRADECHECKINTERVAL ) < now )
	{
		kDebug() << ". sending checkDistroUpgrade";
		emit checkDistroUpgrade();
		UpdaterSettings::self()->setLastDistroUpgradeCheckTime( QDateTime::currentDateTime() );
		UpdaterSettings::self()->writeConfig();
	}
	else
	{
		kDebug() << ". skipping checkDistroUpgrade";
	}
}


void Logics::slotDistroUpgradeNotification()
{
         if ( _distroUpgrades.showNotification() )
                emit distroUpgradeNotification ( _distroUpgrades.upgradesList() );
}


void Logics::startDistroUpgrade()
{
	kDebug() << ". start distro upgrade";

        QProcess* helperprocess = new QProcess(this);
        connect( helperprocess, SIGNAL(finished(int, QProcess::ExitStatus)), SLOT(slotHelperFinished(int, QProcess::ExitStatus)));
        helperprocess->start("kdesu /usr/sbin/wagon");
}

void Logics::startSendSystemProfile()
{
	kDebug() << ". start send system profile";

        QProcess* helperprocess = new QProcess(this);
        connect( helperprocess, SIGNAL(finished(int, QProcess::ExitStatus)), SLOT(slotHelperFinished(int, QProcess::ExitStatus)));
        helperprocess->start("/usr/bin/smoltGui");
}


void Logics::slotDistroUpgrade( const QString &type, const QString &name, const QString &details )
{
        kDebug() << ". distro upgrade available";
        kDebug() << ". type: " << type;
        kDebug() << ". name: " << name;
        kDebug() << ". details: " << details;


        _distroUpgrades.addUpgrade( details );

        emit distroUpgradeAvailable( true );

        QTimer::singleShot(15*1000, this, SLOT(slotDistroUpgradeNotification()));
}


void Logics::slotSystemProfilePopup()
{
	kDebug() << ". check whether to show system profile popup";

	if (! UpdaterSettings::self()->showSystemProfilePopup() )
	{
		kDebug() << ". don't show system profile popup due to configuration";
		return;
	}

	QFile file( "/usr/share/applications/YaST2/suse_register.desktop" );
        if (! file.exists() ) 
	{
		kDebug() << ". don't show system profile popup because this isn't opensuse";
		return;
	}

	QFile execfile( "/usr/bin/smoltGui" );
        if (! execfile.exists() ) 
	{
		kDebug() << ". don't show system profile popup because smoltGui isn't installed";
		return;
	}

	QFile smoltdummyfile( "/var/run/smolt_do_opensuse_run" );
        if (! smoltdummyfile.exists() ) 
	{
		kDebug() << ". don't show system profile popup because /var/run/smolt_do_opensuse_run dosn't exist";
		return;
	}

	if ( _systemProfilePopupShownThisSession )
	{
		kDebug() << ". don't show system profile popup because it was shown already";
		return;
	}

	// if the system installes updates try again later
	// else show popup
	
	if ( _state != APPLET_INSTALLING )
	{
		kDebug() << ". emit systemProfile signal";
		_systemProfilePopupShownThisSession = true;
		emit systemProfile();
	}
	else
	{
		kDebug() << ". new timer";
		QTimer::singleShot(5*60*1000, this, SLOT(slotSystemProfilePopup()));
	}

}

#include "logics.moc"

