#ifndef LOGICS_H
#define LOGICS_H

#include <QTimer>
#include <QProcess>
#include <solid/device.h>

#include "updater.h"
#include "distroupgrades.h"


class QString;
class QTreeWidget;
class QTreeWidgetItem;
class KConfigDialog;
class QTimer;

class Logics : public QObject
{
	Q_OBJECT

public:

	Logics();
	~Logics();

	// select or deselect patches and packages 
	void selectAllPatches();
	void selectAllDrivers();
	void selectAllPackages();
	void deselectAllPatches();
	void deselectAllPackages();
	void selectRecommendedPatches();
	void selectStandardPatches(); 	// =security and packagemanager
	void deselectRestartPatches();


	// count patches and packages
	int packages();
	int patches();
	int drivers();
	int securityPatches();
	int recommendedPatches();
	int restartPatches();
	int packagemanagerPatches();
	

	// count selected patches and packages
	int selectedRecommendedPatches();
	int selectedSecurityPatches();
	int selectedPatches();
	int selectedPackages();
	int selectedRestartPatches();


	// progress information
	int progress() { return _progress; }
	int progressTotalSteps() { return _progressTotal; }
	QString progressDescription() { return _progressDescription; }


	// functions for system information
	bool systemIsBusy();
	bool systemIsOnBattery();
	bool systemIsConnectedToNetwork() { return true; }	// TODO 


	// the capabilites our plugin provides
        Updater::Capabilities updaterCapabilities() { return _caps; }
	void setUpdaterCapabilities( Updater::Capabilities caps ) { _caps = caps; }


	// applet state
	int state() { return _state; } 
	QString stateDescription() { return _stateDescription; }


	// ListViews that contain or patches and packages, shown in the detail window
	void setPatchList( QTreeWidget* patchList ) { _patchList = patchList; }
	void setPackageList( QTreeWidget* packageList) { _packageList = packageList; }
	void setDriverList( QTreeWidget* driverList) { _driverList = driverList; }



	// Debugging
	void setDebugLogging(bool logging) { _debugLogging = logging; }


	// Trigger actions in the plugin
	void checkForUpdates( bool force = false );
	void startInstall( bool force = false );
	void installDrivers();
	void configurePlugin();
	void getDescription( QTreeWidgetItem *resolvable, int type );
	void checkForDistroUpgrade();

	// start a distro upgrade
	void startDistroUpgrade();

	// send hardware information 
	void startSendSystemProfile();

	// start configuration to add an update repo
	void startConfigureUpdateRepo();		

	// start the manual update installtion
	void startManualInstallation();

	// shows the applets's config dialog 
	void configureApplet();

	// sets the initial applet state
	// use slotCheckFinished() instead 
	void setState( int state, const QString& desc = "" );
        /**
         * dump the state on a stream
         */
        QDebug debugStateOn( QDebug str ) const;

	void addDummyPatch(int state);

private:

        Updater::Capabilities _caps;
	QTreeWidget* _patchList;
	QTreeWidget* _packageList;
	QTreeWidget* _driverList;
	int _state;
	bool _debugLogging;
	QString _stateDescription;
	KConfigDialog* _configDialog;
	QTimer _checkTimer;

	int _progress;
	int _progressTotal;
	QString _progressDescription;

	void createListViews();
	void writeAutostartFile();
	void disableAutoSuspend(bool value);
	bool systemReady();
	void dumpUpdateLists();
	QStringList driversStringList();

	bool _skipAutoCheck;
	QList<Solid::Device> _lastHardware;
	QStringList _messages;
	bool _systemProfilePopupShownThisSession;

	DistroUpgrades _distroUpgrades;

	
signals:

	// signals sent to the ui
	void selectionChanged();
	void stateChanged();
	void progressChanged();
	void configurationChanged();
	void resolvableDescription( const QString&, int type );
	void message(const QString&);
        void distroUpgradeAvailable( bool );
        void distroUpgradeNotification( const QStringList &text );


	// signals sent to the plugin
	void checkUpdates( QTreeWidget*, QTreeWidget*, QTreeWidget* );
	void checkDrivers( QTreeWidget*, QTreeWidget*, QTreeWidget* );
	void checkFirmware( QTreeWidget*, QTreeWidget*, QTreeWidget* );
	void install();
	void configure();
	void getResolvableDescription( QTreeWidgetItem *resolvable, int type );
	void stopPackageManagementDaemon();
	void checkDistroUpgrade();
	void systemProfile();

public slots:
	// slots called by the plugin
        void slotAppletError( const UpdaterError &error );
	void slotPluginError( const QString &desc );
	void slotAppletError( const QString &desc );
	void slotAppletMessage( const QString &desc );
	void slotProgress( int, int, const QString &desc );
	void slotPluginNeedsConfiguration( int cause );
	void slotCheckFinished(); 
	void slotInstallFinished(); 
	void slotResolvableDescription( const QString & desc, int type );
	void slotCheckFirmware();
	void slotDistroUpgrade( const QString &type, const QString &name, const QString &details );
        void slotDistroUpgradeNotification();



private slots:
	void slotConfigChanged();
	void slotAutoCheckForUpdates();
	void slotAutoCheckForUpdatesCauseNotReady();
	void slotHelperFinished(int exitCode, QProcess::ExitStatus exitStatus);
	void slotDeviceAdded( const QString & );
	void slotClearHardwareList();
	void slotMessageCollection();
	void slotSystemProfilePopup();

};



#endif
