#include <stdlib.h>
#include <QTabWidget>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QMenu>
#include <QTreeWidget>
#include <QLabel>
#include <QTimer>
#include <QLayout>
#include <KAction>
#include <KStandardAction>
#include <KLocale>
#include <KApplication>
#include <KActionCollection>
#include <KConfig>
#include <KDebug>
#include <KTextEdit>
#include <KPushButton>
#include <KApplication>
#include <KGlobal>
#include <KIconLoader>

#include "simpleview.h"
#include "constants.h"
#include "headerwidget.h"
#include "mainwindow.h"
#include "kupdateapplet_settings.h"
#include "updater.h"
#include "trayicon.h"
#include "configwidget.h"
#include "ksystemtrayicon.h"
#include "logics.h"
#include "updater.h"

MainWindow::MainWindow( Logics *l )
	: KMainWindow( 0 )
	, logics( l )
{
	initGUI();

	slotConfigChanged();
	slotStateChanged();
	hide();
}


void MainWindow::initGUI()
{
	kDebug() << "init gui..";


	// This list is never shown
	// It is simmilar tor patch and packagelist 
	// it contails a list of drivers that need to
	// be installed if new hardware has been attached
	driverList = new QTreeWidget(this);
        driverList->hide();

	driverList->setSelectionMode( QAbstractItemView::SingleSelection );

        QStringList headers;
        
	headers << i18n( "Name" );
	//FIXME driverList->setColumnWidthMode( COLUMN_NAME, QTreeWidget::Maximum );
	headers << i18n( "Summary" );
	headers << i18n( "Type" );
	headers << i18n( "New Version");
	headers << i18n( "Catalog" );
	headers << i18n( "Restart" );
	headers << "RestartHidden";
        driverList->setHeaderLabels(headers);

	centralWidget = new QWidget( this );
	setCentralWidget( centralWidget );

	setWindowIcon( KIconLoader::global()->loadIcon( PROGRAM_NAME, KIconLoader::Small ) );

	mainBox = new QVBoxLayout( centralWidget );
	header = new HeaderWidget( centralWidget );

	// ------[ Patches - Tab ]---

	tabwidget = new QTabWidget( centralWidget );

	patchWidget = new QWidget( centralWidget );
	patchLabel = new QLabel( patchWidget );
	patchLabel->setText( i18n( "The following official patches for your distribution either fix security holes or errors in the software." ) );
	QVBoxLayout *patchBox = new QVBoxLayout( patchWidget );
	patchDescription = new KTextEdit( patchWidget );
	patchDescription->setReadOnly( true );
	tabwidget->addTab( patchWidget, i18n( "Patches" ) );

	patchBox->addWidget( patchLabel, 0, 0 );

	//-----
	patchList = new QTreeWidget(patchWidget);
	patchList->setSelectionMode( QAbstractItemView::SingleSelection );
        patchList->setRootIsDecorated(false);

        headers = QStringList();
        headers << QString();
	headers << i18n( "Name" );
	// FIXME patchList->setColumnWidthMode( COLUMN_NAME, QTreeWidget::Maximum );
	headers << i18n( "Summary" );
	headers << i18n( "Type" );
	headers << i18n( "New Version" );
	headers << i18n( "Repository" );
	headers << i18n( "Restart" );
	headers << "Id";
	headers << "RestartHidden";

        patchList->setHeaderLabels(headers);
        patchList->hideColumn(COLUMN_RESTART_HIDDEN);
        patchList->hideColumn(COLUMN_ID);

	//---------

	patchBox->addWidget( patchList, 0, 0 );

	QWidget* patchSelectionButtonArea = new QWidget ( patchWidget);

	patchSelectAllButton = new KPushButton( i18n( "Select All Patches" ), patchSelectionButtonArea );
	patchClearSelectionButton = new KPushButton( i18n( "Clear Patch Selection" ), patchSelectionButtonArea );

	patchSelectionButtonsLayout = new QHBoxLayout( patchSelectionButtonArea );
	patchSelectionButtonsLayout->addWidget( patchSelectAllButton, false, Qt::AlignLeft );
	patchSelectionButtonsLayout->addWidget( patchClearSelectionButton, false, Qt::AlignRight );

	patchBox->addWidget( patchSelectionButtonArea, 0, 0 );
	patchBox->addWidget( patchDescription, 0, 0 );

	patchBox->setSpacing( 10 );
	patchBox->setMargin( 10 );



	// ------[ Packages - Tab ]---

	packageWidget = new QWidget( tabwidget );
	packageLabel = new QLabel( packageWidget );
	packageLabel->setText( i18n( "The following list contains new versions for software packages installed on your system.<br>"
                                 "These updates are provided by the opensource community." ) );
	QVBoxLayout *packageBox = new QVBoxLayout( packageWidget );
	packageDescription = new KTextEdit( packageWidget );
	packageDescription->setReadOnly( true );
	packageBox->addWidget( packageLabel, 0, 0 );

	//----
	packageList = new QTreeWidget(packageWidget);
	packageList->setSelectionMode( QAbstractItemView::SingleSelection );
        packageList->setRootIsDecorated(false);

        headers = QStringList();
        headers << QString();
	headers << i18n( "Name" );
	// FIXME packageList->setColumnWidthMode( COLUMN_NAME, QTreeWidget::Maximum );
	headers << i18n( "Summary" );
	headers << i18n( "Type" );
	headers << i18n( "New Version" );
	headers << i18n( "Catalog" );
	headers << i18n( "Restart" );
	headers << "RestartHidden";

        packageList->setHeaderLabels(headers);
        packageList->hideColumn(COLUMN_RESTART_HIDDEN);
        packageList->hideColumn(COLUMN_ID);
        
	//---
	packageBox->addWidget( packageList, 0, 0 );
	
	QWidget* packageSelectionButtonArea = new QWidget ( packageWidget);
	packageSelectionButtonsLayout = new QHBoxLayout(packageSelectionButtonArea);

	packageSelectAllButton = new KPushButton( i18n( "Select All Packages" ), packageSelectionButtonArea );
	packageClearSelectionButton = new KPushButton( i18n( "Clear Package Selection" ), packageSelectionButtonArea );
	packageSelectionButtonsLayout->addWidget( packageSelectAllButton, false, Qt::AlignLeft );
	packageSelectionButtonsLayout->addWidget( packageClearSelectionButton, false, Qt::AlignRight );

	packageBox->addWidget( packageSelectionButtonArea, 0, 0 );
	packageBox->addWidget( packageDescription, 0, 0 );
	tabwidget->addTab( packageWidget, i18n( "Upgrades" ) );


	// ----------------
	logics->setPatchList(patchList);
	logics->setPackageList(packageList);
	logics->setDriverList(driverList);
	// -----------------

	mainBox->addWidget( header, 0, 0 );
	mainBox->addWidget( tabwidget, 0, 0 );

	QWidget *buttonsBox= new QWidget(this);

	buttonsLayout = new QHBoxLayout(buttonsBox);
        mainBox->addWidget(buttonsBox);
        
	configureButton = new KPushButton( i18n( "Add/Remove Update Repositories..." ), buttonsBox );
	installButton = new KPushButton( KStandardGuiItem::ok(), buttonsBox );
	installButton->setText( i18n( "Install" ) );
	cancelButton = new KPushButton( KStandardGuiItem::cancel(), buttonsBox );

	buttonsLayout->addWidget( configureButton, false, Qt::AlignLeft );
	buttonsLayout->addSpacing( 250 );
	buttonsLayout->addStretch();
	buttonsLayout->addWidget( installButton, false, Qt::AlignRight );
	buttonsLayout->addSpacing( 10 );
	buttonsLayout->addWidget( cancelButton, false, Qt::AlignRight );

	packageBox->setSpacing( 10 );
	packageBox->setMargin( 10 );
	mainBox->setSpacing( 10 );
	mainBox->setMargin( 10 );

	connect( configureButton, SIGNAL( clicked() ), this, SLOT( slotServerButtonClicked() ) );
	connect( installButton, SIGNAL( clicked() ), this, SLOT( slotInstallButtonClicked() ) );
	connect( cancelButton, SIGNAL( clicked() ), this, SLOT( hide() ) );
	connect( patchSelectAllButton, SIGNAL( clicked() ), this, SLOT( slotPatchSelectButtonClicked() ) );
	connect( packageSelectAllButton, SIGNAL( clicked() ), this, SLOT( slotPackageSelectButtonClicked() ) );
	connect( patchClearSelectionButton, SIGNAL( clicked() ), this, SLOT( slotPatchClearButtonClicked() ) );
	connect( packageClearSelectionButton, SIGNAL( clicked() ), this, SLOT( slotPackageClearButtonClicked() ) );
	connect( patchList, SIGNAL( currentItemChanged( QTreeWidgetItem*, QTreeWidgetItem* ) ), this, SLOT( slotPatchSelected( QTreeWidgetItem* ) ) );
	connect( packageList, SIGNAL( currentItemChanged( QTreeWidgetItem*,QTreeWidgetItem* ) ), this, SLOT( slotPackageSelected( QTreeWidgetItem* )));
	connect( patchList, SIGNAL( itemClicked( QTreeWidgetItem*, int ) ), this, SLOT( slotPatchSelected( QTreeWidgetItem* ) ) );
	connect( packageList, SIGNAL( itemClicked( QTreeWidgetItem*, int ) ), this, SLOT( slotPackageSelected( QTreeWidgetItem* ) ) );

	resize( 400, 500 );
	setWindowIcon( UserIcon( TRAY_ICON_GREEN ) );
	setCaption( i18n( "Available Updates" ) );

	slotConfigChanged();

	hide();
}


void MainWindow::updateButtons()
{
	// when updates are selected for installation, enable "Install" button
	if (( ( logics->updaterCapabilities() 
          & Updater::CanSelectIndividualUpdates )  &&
			( logics->selectedPatches() + logics->selectedPackages() ) > 0 )  ||
        ( ! (logics->updaterCapabilities() & Updater::CanSelectIndividualUpdates ) &&
                          ( patchList->topLevelItemCount() + packageList->topLevelItemCount() ) > 0 )	)
	{
		installButton->setDisabled( false );
	}
	else
	{
		installButton->setDisabled( true );
	}
}


void MainWindow::slotConfigChanged()
{
	// decide whether to show select all/none buttons
        if ( logics->updaterCapabilities()
         & Updater::CanSelectIndividualUpdates )
	{
		patchSelectAllButton->show();
		patchClearSelectionButton->show();

		packageSelectAllButton->show();
		packageClearSelectionButton->show();
	}
	else
	{
		patchSelectAllButton->hide();
		patchClearSelectionButton->hide();

		packageSelectAllButton->hide();
		packageClearSelectionButton->hide();
	}

	// decide whether to show packages tab
	if ( ( logics->updaterCapabilities()
               & Updater::CanProvidePackages )  
             && UpdaterSettings::self()->showPackages() )
	{
                tabwidget->removeTab(tabwidget->indexOf(packageWidget));
		tabwidget->removeTab(tabwidget->indexOf(patchWidget));

		tabwidget->addTab( patchWidget, i18n( "Patches" ) );
		tabwidget->addTab( packageWidget, i18n( "Upgrades" ) );
		tabwidget->setCurrentIndex( 1 );
		tabwidget->setCurrentIndex( 0 );
	}
	else
	{
                tabwidget->removeTab(tabwidget->indexOf(packageWidget));
		tabwidget->removeTab(tabwidget->indexOf(patchWidget));

		tabwidget->addTab( patchWidget, i18n( "Patches" ) );
	}
}


void MainWindow::slotStateChanged() 
{
	slotConfigChanged();
	updateButtons();
}


void MainWindow::slotGotDescription( QString description, int resolvableType )
{
    if ( resolvableType == RESOLVABLE_PATCH )
        patchDescription->setPlainText( description );
    else
        packageDescription->setPlainText( description );
}


void MainWindow::slotServerButtonClicked()
{
	logics->configurePlugin();
}


void MainWindow::slotInstallButtonClicked()
{
	logics->startInstall();
	hide();
}


bool MainWindow::queryClose()
{
	//Make the main window hide, not quit
	if ( !kapp->sessionSaving() )
	{
		hide();
		return false;
	}
	return true;
}


void MainWindow::slotPatchSelectButtonClicked()
{
	logics->selectAllPatches();
	updateButtons();
}

void MainWindow::slotPackageSelectButtonClicked()
{
	logics->selectAllPackages();
	updateButtons();
}


void MainWindow::slotPatchClearButtonClicked()
{
	logics->deselectAllPatches();
	updateButtons();
}


void MainWindow::slotPackageClearButtonClicked()
{
	logics->deselectAllPackages();
	updateButtons();
}


void MainWindow::slotPackageSelected( QTreeWidgetItem *package )
{
	updateButtons();
	logics->getDescription( package, RESOLVABLE_PACKAGE );
}


void MainWindow::slotPatchSelected( QTreeWidgetItem *patch )
{
	updateButtons();
	logics->getDescription( patch, RESOLVABLE_PATCH );
}


void MainWindow::slotPatchSelectionChanged()
{
	updateButtons();
}


void MainWindow::slotPackageSelectionChanged()
{
	updateButtons();
}


void MainWindow::slotSelectionChanged()
{
	updateButtons();
}


#include "mainwindow.moc"
