#ifndef _MAIN_WINDOW_H_
#define _MAIN_WINDOW_H_

#include <QObject>
#include <QLabel>
#include <QTabWidget>

#include <KSystemTrayIcon>
#include <KXmlGuiWindow>
#include <KConfigDialog>

#include "simpleview.h"
#include "updater.h"

class TrayIcon;
class KTextEdit;
class KPushButton;
class QTreeWidget;
class QTreeWidgetItem;
class QTimer;
class QVBoxLayout;
class QHBoxLayout;
class Updater;
class HeaderWidget;
class OSUpdaterPopup;
class KAction;
class SimpleView;
class Logics;

// Class to represent our MainWindow (a.k.a. detailed view)
class MainWindow : public KMainWindow
{
Q_OBJECT
public:

        Updater::Capabilities getUpdaterCapabilities() const;
	MainWindow(Logics *logics);

public slots:

	// updates the applet to the specified state
	void slotStateChanged();

	// selected patches or patches have changed
	void slotSelectionChanged();

	// reacts to changed configuration
	void slotConfigChanged();

	// gets the description from the backend. 
	void slotGotDescription(QString desc, int resolvableType);


private slots:

	void slotPackageSelected(QTreeWidgetItem *package);
	void slotPatchSelected(QTreeWidgetItem *path);
	void slotPackageSelectionChanged(); 
	void slotPatchSelectionChanged(); 
	void slotServerButtonClicked();
	void slotInstallButtonClicked();
	void slotPatchSelectButtonClicked();
	void slotPackageSelectButtonClicked();
	void slotPatchClearButtonClicked();
	void slotPackageClearButtonClicked();
	bool queryClose();


private:
	void initGUI();
	void updateButtons();
	void updateTabs();
	
	// our business logic
	Logics* logics;

	// lists containing patches and packages
	QTreeWidget *patchList;
	QTreeWidget *packageList;
	QTreeWidget *driverList;

	// gui elements
	KTextEdit *patchDescription;
	KTextEdit *packageDescription;
	HeaderWidget *header;

	KPushButton *installButton;
	KPushButton *cancelButton;
	KPushButton *patchSelectAllButton;
	KPushButton *patchClearSelectionButton;
	KPushButton *packageInstallButton;
	KPushButton *packageCancelButton;
	KPushButton *packageSelectAllButton;
	KPushButton *packageClearSelectionButton;
	KPushButton *configureButton;

	QVBoxLayout *mainBox;
	QHBoxLayout *patchSelectionButtonsLayout;
	QHBoxLayout *packageSelectionButtonsLayout;
	QHBoxLayout *buttonsLayout;
	QHBoxLayout *packageButtonsLayout;
	QWidget *centralWidget;
	QTabWidget *tabwidget;
	QWidget *patchWidget;
	QWidget *packageWidget;
	
	QLabel *packageLabel;
	QLabel *patchLabel;

};


#endif

