#include <QObject>
#include <QLabel>
#include <QPushButton>
#include <QLayout>
#include <QDateTime>
#include <kdebug.h>
#include <klocale.h>
#include <knotification.h>
#include <kpassivepopup.h>

#include "kupdateapplet_settings.h"
#include "constants.h"
#include "updater.h"
#include "logics.h"
#include "notification.h"


Notification::Notification( Logics* l, QWidget* w)
	: _logics ( l )
	, _widget ( w )
	, _lastPopup (QDateTime())
{
}

Notification::~Notification()
{
}

void Notification::slotProgressChanged()
{
	int progress = 0;
	
	if ( _logics->progressTotalSteps() != 0 )
		progress = _logics->progress() / _logics->progressTotalSteps() ;

	_logics->progressDescription();

	QString text = QString("%1%: %2").arg(progress).arg(_logics->progressDescription());


	KNotification *notification= new KNotification ( "progress", _widget );
	notification->setText( text );
	notification->setActions( QStringList( i18n( "more ..." ) ) );
	connect(notification, SIGNAL(activated(unsigned int )), this , SLOT(slotPopupActivated(unsigned int)) );
	notification->sendEvent();

}


void Notification::slotMessage(const QString &message)
{
	KNotification *notification= new KNotification ( "appletmessage", _widget, KNotification::Persistent );
	notification->setText( message );
	notification->sendEvent();
}

void Notification::slotStateChanged()
{
	if (_logics->state() ==  APPLET_PROBLEM)
	{
		KNotification *notification= new KNotification ( "appletproblem", _widget );
		notification->setText( i18n("An error occured") );
		notification->setActions( QStringList( i18n( "more ..." ) ) );
		connect(notification, SIGNAL(activated(unsigned int )), this , SLOT(slotPopupActivated(unsigned int)) );
		notification->sendEvent();
	}

	else if ( _logics->state() ==  APPLET_PKGMGR_UPDATES ||
		  _logics->state() ==   APPLET_UPDATES || 
		  _logics->state() ==  APPLET_CRITICAL_UPDATES )
	{
		if ( _lastPopup.addSecs( 30 ) < QDateTime::currentDateTime() )
		{

			KNotification *notification= new KNotification ( "newupdates", _widget );
			notification->setText( i18n("New software updates for your system are available.") );
			notification->setActions( QStringList( i18n( "more ..." ) ) );
			connect(notification, SIGNAL(activated(unsigned int )), this , SLOT(slotPopupActivated(unsigned int)) );
			notification->sendEvent();
			_lastPopup = QDateTime::currentDateTime();
		}
	}
	
	else if (_logics->state() ==  APPLET_NO_UPDATE_REPO )
	{
		KNotification *notification= new KNotification ( "noupdaterepo", _widget );
		notification->setText( i18n("Please configure an update repository.") );
		notification->setActions( QStringList( i18n( "more ..." ) ) );
		connect(notification, SIGNAL(activated(unsigned int )), this , SLOT(slotPopupActivated(unsigned int)) );
		notification->sendEvent();
	}

	else if (_logics->state() ==  APPLET_NEW_HARDWARE )
	{
		KNotification *notification= new KNotification ( "newhardware", _widget );
		notification->setText( i18n("New hardware has been attached.") );
		notification->setActions( QStringList( i18n( "more ..." ) ) );
		connect(notification, SIGNAL(activated(unsigned int )), this , SLOT(slotPopupActivated(unsigned int)) );
		notification->sendEvent();
	}
}

void Notification::slotPopupActivated(unsigned int number)
{
	Q_UNUSED(number);
	emit showSimpleView();
}

void Notification::slotStartDistroUpgrade(unsigned int number)
{
	kDebug() << "start distro upgrade";
	Q_UNUSED(number);
	_logics->startDistroUpgrade();
}


void Notification::slotDistroUpgradeNotification( const QStringList &text )
{

	kDebug() << "distro upgrade";

	KNotification *notification= new KNotification ( "distroupgrade", _widget, KNotification::Persistent );
	notification->setText( i18n("A distribution upgrade is available:") + "<br>" + text.join("<br>") );
	notification->setActions( QStringList( i18n( "Upgrade ..." ) ) );
	connect(notification, SIGNAL(activated(unsigned int )), this , SLOT(slotStartDistroUpgrade(unsigned int)) );
	notification->sendEvent();
}

void Notification::slotSystemProfile()
{

	kDebug() << "system profile";

	KNotification *notification= new KNotification ( "systemprofile", _widget, KNotification::Persistent );
	notification->setText( i18n("Driver development is prioritized based on hardware popularity.<br>"
				"Please send your system profile to influence this work.") );
	QStringList actions;
	actions << i18n("Send now");	
	actions << i18n("Don't ask again");	
	notification->setActions( actions );
	connect(notification, SIGNAL(activated(unsigned int )), this , SLOT(slotSystemProfileActivated(unsigned int)) );
	notification->sendEvent();
}

void Notification::slotSystemProfileActivated(unsigned int number)
{
	kDebug() << "system profile activated";

	// showing the popup once is enough
	UpdaterSettings::self()->setShowSystemProfilePopup(false);
	UpdaterSettings::self()->writeConfig();

	if ( number == 1 )
	{
		kDebug() << "send system profile";
		_logics->startSendSystemProfile();
	}
}



#include "notification.moc"

