/*
 * Copyright (C) 2007 Adrien Bustany <madcat@mymadcat.com>
 *
 * Licensed under the GNU General Public License Version 2
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#include "qpackagekitconnector.h"

QMap<int, QString> QPackageKitConnector::Role::stringValues = QMap<int, QString>();
void QPackageKitConnector::Role::construct () {
	stringValues[Cancel] = "cancel";
	stringValues[Resolve] = "resolve";
	stringValues[Rollback] = "rollback";
	stringValues[Get_depends] = "get-depends";
	stringValues[Get_update_detail] = "get-update-detail";
	stringValues[Get_description] = "get-description";
	stringValues[Get_requires] = "get-requires";
	stringValues[Get_updates] = "get-updates";
	stringValues[Search_details] = "search-details";
	stringValues[Search_file] = "search-file";
	stringValues[Search_group] = "search-group";
	stringValues[Search_name] = "search-name";
	stringValues[Refresh_cache] = "refresh-cache";
	stringValues[Update_system] = "update-system";
	stringValues[Remove_package] = "remove-package";
	stringValues[Install_package] = "install-package";
	stringValues[Install_file] = "install-file";
	stringValues[Update_package] = "update-package";
	stringValues[Get_repo_list] = "get-repo-list";
	stringValues[Repo_enable] = "repo-enable";
	stringValues[Repo_set_data] = "repo-set-data";
	stringValues[Unknown] = "unknown";
}

QString QPackageKitConnector::Role::toString(RoleEnum e) {
	if(stringValues.size() == 0) construct();
	return stringValues[e];
}

QPackageKitConnector::Role::RoleEnum QPackageKitConnector::Role::fromString(QString s) {
	if(stringValues.size() == 0) construct();
	return (RoleEnum)stringValues.key(s);
}

QMap<int, QString> QPackageKitConnector::Status::stringValues = QMap<int, QString>();
void QPackageKitConnector::Status::construct () {
	//Status enum
	stringValues[Setup] = "setup";
	stringValues[Wait] = "wait";
	stringValues[Query] = "query";
	stringValues[Remove] = "remove";
	stringValues[Refresh_cache] = "refresh-cache";
	stringValues[Download] = "download";
	stringValues[Install] = "install";
	stringValues[Update] = "update";
	stringValues[Unknown] = "unknown";
}

QString QPackageKitConnector::Status::toString(StatusEnum e) {
	if(stringValues.size() == 0) construct();
	return stringValues[e];
}

QPackageKitConnector::Status::StatusEnum QPackageKitConnector::Status::fromString(QString s) {
	if(stringValues.size() == 0) construct();
	return (StatusEnum)stringValues.key(s);
}

QMap<int, QString> QPackageKitConnector::Exit::stringValues = QMap<int, QString>();
void QPackageKitConnector::Exit::construct () {
	stringValues[Success] = "success";
	stringValues[Failed] = "failed";
	stringValues[Canceled] = "canceled";
	stringValues[Unknown] = "unknown";
}

QString QPackageKitConnector::Exit::toString(ExitEnum e) {
	if(stringValues.size() == 0) construct();
	return stringValues[e];
}

QPackageKitConnector::Exit::ExitEnum QPackageKitConnector::Exit::fromString(QString s) {
	if(stringValues.size() == 0) construct();
	return (ExitEnum)stringValues.key(s);
}

QMap<int, QString> QPackageKitConnector::Filter::stringValues = QMap<int, QString>();
void QPackageKitConnector::Filter::construct () {
	stringValues[Development] = "development";
	stringValues[Installed] = "installed";
	stringValues[Gui] = "gui";
	stringValues[Normal] = "~devel";
	stringValues[Available] = "~installed";
	stringValues[Text] = "~gui";
	stringValues[Unknown] = "unknown";
}

QString QPackageKitConnector::Filter::toString(FilterEnum e) {
	if(stringValues.size() == 0) construct();
	return QPackageKitConnector::Filter::stringValues[e];
}

QPackageKitConnector::Filter::FilterEnum QPackageKitConnector::Filter::fromString(QString s) {
	if(stringValues.size() == 0) construct();
	return (FilterEnum)stringValues.key(s);
}


typedef enum {
    PK_ERROR_ENUM_OOM,
    PK_ERROR_ENUM_NO_NETWORK,
    PK_ERROR_ENUM_NOT_SUPPORTED,
    PK_ERROR_ENUM_INTERNAL_ERROR,
    PK_ERROR_ENUM_GPG_FAILURE,
    PK_ERROR_ENUM_PACKAGE_ID_INVALID,
    PK_ERROR_ENUM_PACKAGE_NOT_INSTALLED,
    PK_ERROR_ENUM_PACKAGE_NOT_FOUND,
    PK_ERROR_ENUM_PACKAGE_ALREADY_INSTALLED,
    PK_ERROR_ENUM_PACKAGE_DOWNLOAD_FAILED,
    PK_ERROR_ENUM_GROUP_NOT_FOUND,
    PK_ERROR_ENUM_GROUP_LIST_INVALID,
    PK_ERROR_ENUM_DEP_RESOLUTION_FAILED,
    PK_ERROR_ENUM_FILTER_INVALID,
    PK_ERROR_ENUM_CREATE_THREAD_FAILED,
    PK_ERROR_ENUM_TRANSACTION_ERROR,
    PK_ERROR_ENUM_TRANSACTION_CANCELLED,
    PK_ERROR_ENUM_NO_CACHE,
    PK_ERROR_ENUM_REPO_NOT_FOUND,
    PK_ERROR_ENUM_CANNOT_REMOVE_SYSTEM_PACKAGE,
    PK_ERROR_ENUM_PROCESS_KILL,
    PK_ERROR_ENUM_FAILED_INITIALIZATION,
    PK_ERROR_ENUM_FAILED_FINALISE,
    PK_ERROR_ENUM_FAILED_CONFIG_PARSING,
    PK_ERROR_ENUM_CANNOT_CANCEL,
    PK_ERROR_ENUM_CANNOT_GET_LOCK,
    PK_ERROR_ENUM_NO_PACKAGES_TO_UPDATE,
    PK_ERROR_ENUM_CANNOT_WRITE_REPO_CONFIG,
    PK_ERROR_ENUM_LOCAL_INSTALL_FAILED,
    PK_ERROR_ENUM_BAD_GPG_SIGNATURE,
    PK_ERROR_ENUM_MISSING_GPG_SIGNATURE,
    PK_ERROR_ENUM_CANNOT_INSTALL_SOURCE_PACKAGE,
    PK_ERROR_ENUM_REPO_CONFIGURATION_ERROR,
    PK_ERROR_ENUM_NO_LICENSE_AGREEMENT,
    PK_ERROR_ENUM_FILE_CONFLICTS,
    PK_ERROR_ENUM_REPO_NOT_AVAILABLE,
    PK_ERROR_ENUM_INVALID_PACKAGE_FILE,
    PK_ERROR_ENUM_PACKAGE_INSTALL_BLOCKED,
    PK_ERROR_ENUM_PACKAGE_CORRUPT,
    PK_ERROR_ENUM_UNKNOWN
} PkErrorCodeEnum;
