/*
   Copyright (C) 2006,2007.2008 Novell Inc

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#ifndef _PACKAGEKIT_UPDATER_H_
#define _PACKAGEKIT_UPDATER_H_

#include "updater.h"
#include "pklicensedialog.h"
#include <QtCore/QList>
#include <QProcess>
#include <QMutex>
#include <QStringList>
#include "qpackagekitconnector.h"
class QPackageKitConnector;

class PackageKitUpdater : public Updater
{
	Q_OBJECT
public:

	PackageKitUpdater( QObject *parent, const QStringList&);
	virtual ~PackageKitUpdater() {};

	virtual Updater::Capabilities capabilities() const;
	virtual void shutdown();

public slots:

	// Slot which gets called when we need to throw up the add/remove server dialog.
	void slotConfigureUpdater();

	// Slot which gets called when we need to start an update. 
	// Collect our update info from the list we got passed 
	// in populateList and go. 
	void slotStartInstall();

	// Slot which gets called when we need to check for updates 
	// and load up the list. 
	void slotPopulateLists(QTreeWidget *patchList, QTreeWidget *packageList, QTreeWidget *driverList);
	void slotCheckDrivers(QTreeWidget *patchList, QTreeWidget *packageList, QTreeWidget *driverList);
	void slotCheckFirmware(QTreeWidget *patchList, QTreeWidget *packageList, QTreeWidget *driverList);

	// Slot which gets called when the user selects an update. 
	// We are expected to write a description of the update and pass it back.
	void slotResolvableSelected(QTreeWidgetItem* item, int);
	void slotRequireRestart(const QString& type, const QString& details);
	void slotMessage(const QString& type, const QString& details);


	// slots for communication with packagekit
	void slotNewPackage(const QString& info, const QString& package_id, const QString& summary);
	void slotGotDetails(const QString &package_id, const QString &licence, const QString &group,
		const QString &detail, const QString &url, qulonglong size); 
	void slotTransactionFinished(QPackageKitConnector::Exit::ExitEnum exitCode, uint runtime);
	void slotNoProgressInfo();
	void slotUpdateProgress(uint percentage, uint subpercentage, uint elapsed, uint remaining); 
	void slotError(const QString &message);

        /**
         * this slot is connected to the transaction finished
         * slot from the connector which emits a signal if the
         * transaction was updates */
	void slotGetUpdatesFinished();	
	void slotInstallFinished();	
        void slotDriverSearchFinished();

	void slotShowEula(const QString& eula_id, const QString& package_id, const QString& vendor_name,
		const QString& license_agreement);

	void slotRepoSignatureRequired(const QString &package_id, const QString &repository_name,
                const QString  &key_url, const QString &key_userid, const QString &key_id,
                const QString &key_fingerprint, const QString &key_timestamp, const QString &type);

	void slotStatusChanged( const QString &status );
	void slotRepoDetail(const QString &repo_id, const QString &desc, bool enabled);
	void slotUpdateDetail(const QString &package_id, const QString &updates, const QString &obsoletes, const QString &vendor_url,
                const QString &bugzilla_url, const QString &cve_url, const QString &restart, const QString &update_text,
                const QString &changelog, const QString &state, const QString &issued, const QString &updated );

	void slotCheckDistroUpgrade();
	void slotDistroUpgrade( const QString &type, const QString &name, const QString &details );
	void slotUpdaterRepoSearchFinished();


protected slots:
	void slotShowLog();
	void slotStopPackageManagementDaemon();
	void slotWaitForDetailsComplete();


private:
	void doCheckForUpdates();
	void doCheckForDrivers();
	void doCheckForFirmware();
	void doCheckForUpdateRepos();
	QPackageKitConnector* pkConnector ;

	QTreeWidget *_patchList;
	QTreeWidget *_packageList;
	QTreeWidget *_driverList;

	int _packageDescriptionCount;
	void updateStatus();

	bool _securityAvailable;
	bool _errors;
	bool _softerrors;	// errors that can be solved by accepting eula and trying again 
	bool _updateRepoAvailable;

	QString _lastStatus;
	PkLicenseDialog *licenseDialog;

	bool checkRegistrationRan();
	bool _driverSearch;

	void searchFirmware();
        
        // mutext shared with functions
        // that get packages async
        QMutex *_get_packages_mutex;

	QStringList _packageDetailsList;
	int _missing_details;
};

#endif

// vim: ts=2 noet sw=2
