#include <kdebug.h>
#include <KLocale>

#include "qpackagekitinterface.h"
#include "qpackagekittransactioninterface.h"
#include "qpackagekitconnector.h"
#include "polkitclient.h"
#include "pkconstants.h"
#include "pkconstants.h"

QPackageKitConnector::QPackageKitConnector( QObject *parent ) 
	: QObject( parent )
	, _lastAction("none")
	, _lastActionArg(false)
{
}


void QPackageKitConnector::disconnectFromInterface(QPackageKitTransactionInterface* transactioninterface)
{
	disconnect(transactioninterface, 
		SIGNAL( Package(const QString, const QString, const QString) ), 
		this,
		SLOT( proxy_Package(const QString, const QString, const QString) )
	);

	disconnect(transactioninterface,
		SIGNAL( Finished(const QString, uint) ),
		this,
		SLOT( proxy_Finished(const QString, uint) )
	);

	disconnect(transactioninterface,
		SIGNAL( ProgressChanged(uint, uint, uint, uint) ),
		this,
		SLOT( proxy_ProgressChanged(uint,uint, uint, uint) )
	);

	disconnect(transactioninterface,
		SIGNAL( StatusChanged(const QString &) ),
		this,
		SLOT( proxy_StatusChanged(const QString &) )
	);

	disconnect(transactioninterface,
             SIGNAL( Transaction(const QString &, const QString &, bool, const QString &, uint, const QString &) ),
             this,
             SLOT(proxy_Transaction(const QString &, const QString &, bool, const QString &, uint, const QString &) )
	);

	disconnect(transactioninterface, 
		SIGNAL( Details(const QString&, const QString&, const QString&,
				 const QString&, const QString&, qulonglong)),
		this,
		SLOT( proxy_Details(const QString&, const QString&, const QString&,
				const QString&,	const QString&, qulonglong))
	);

	disconnect (transactioninterface,
		SIGNAL( UpdateDetail(const QString&, const QString&, const QString&, const QString&, 
			const QString&, const QString&, const QString&, const QString&,
			const QString&, const QString&, const QString&, const QString&)),

		this,
		SLOT( proxy_UpdateDetail(const QString&, const QString&, const QString&, const QString&, 
			const QString&, const QString&, const QString&, const QString&,
			const QString&, const QString&, const QString&, const QString&))
	);


	disconnect(transactioninterface, 
		SIGNAL( ErrorCode(const QString&, const QString& )),
		this,
		SLOT( slotError( const QString&, const QString&))
	);

	
	disconnect(transactioninterface,
		SIGNAL ( RepoSignatureRequired(const QString&,  const QString&,
			const QString&, const QString&, const QString&,
			const QString&, const QString&, const QString&)),
		this,
		SLOT (proxy_RepoSignatureRequired(const QString&, const QString&,
			const QString&, const QString&, const QString &,
			const QString&, const QString&, const QString &))
	);


	disconnect(transactioninterface,
		SIGNAL( EulaRequired(const QString&, const QString&, const QString&,
				const QString&)),
		this,
		SLOT( proxy_EulaRequired(const QString&, const QString&, const QString&,
				const QString&))
	);

	disconnect(transactioninterface,
		SIGNAL (RepoDetail(const QString&, const QString&, bool)),
		this,
		SLOT(proxy_RepoDetail(const QString&, const QString &, bool))
	);

	disconnect(transactioninterface,
		SIGNAL (RequireRestart(const QString&, const QString&)),
		this,
		SLOT(proxy_RequireRestart(const QString&, const QString &))
	);


	disconnect(transactioninterface,
		SIGNAL (Message(const QString&, const QString&)),
		this,
		SLOT(proxy_Message(const QString&, const QString &))
	);

	disconnect(transactioninterface,
                   SIGNAL (DistroUpgrade(const QString&, const QString&, const QString &)),
                   this,
                   SLOT(proxy_DistroUpgrade(const QString&, const QString &, const QString &))
            );

	transactioninterface->deleteLater();

}

void QPackageKitConnector::slotError(const QString& code, const QString& message)
{
        // ignore errors which can be handled by showing a 
        // popup and restart transaction afterwards
        
        if (code == "no-license-agreement" ||
            code == "bad-gpg-signature" ||
            code == "gpg-failure" ||
            code == "missing-gpg-signature" ) 
        { 
       		emit ErrorMessage(QString());
        }
	else
	{
	        emit ErrorMessage("PackageKit Error " + code + ": " + message);	 
	}
}

QPackageKitTransactionInterface* QPackageKitConnector::connectToInterface()
{

	QString tid = getTid();
	if (tid == QString())	// tid is not valid
	{
		emit ErrorMessage( 
			"DBus error : Unable to get transaction id from packagekit."
				 );
		return NULL;
	}

	QPackageKitTransactionInterface *transactioninterface = new QPackageKitTransactionInterface(QString::fromAscii(PK_DBUS_SERVICE),
                                                                                              tid,
                                                                                              QDBusConnection::systemBus() );

	connect(transactioninterface, 
		SIGNAL( Package(const QString, const QString, const QString) ), 
		this,
		SLOT( proxy_Package(const QString, const QString, const QString) )
	);

	connect(transactioninterface,
                SIGNAL( Finished(const QString, uint ) ),
                this,
                SLOT( proxy_Finished(const QString, uint ) )
	);

	connect(transactioninterface,
                SIGNAL( Transaction(const QString &, const QString &, bool, const QString &, uint, const QString &) ),
                SLOT(proxy_Transaction(const QString &, const QString &, bool, const QString &, uint, const QString &) )
	);
  
	connect(transactioninterface,
		SIGNAL( ProgressChanged(uint, uint, uint, uint) ),
		this,
		SLOT( proxy_ProgressChanged(uint,uint, uint, uint) )
	);

	connect(transactioninterface,
		SIGNAL( StatusChanged(const QString &) ),
		this,
		SLOT( proxy_StatusChanged(const QString &) )
	);




	connect(transactioninterface, 
		SIGNAL( Details(const QString&, const QString&, const QString&,
				 const QString&, const QString&, qulonglong)),
		this,
		SLOT( proxy_Details(const QString&, const QString&, const QString&,
				const QString&,	const QString&, qulonglong))
	);

	connect (transactioninterface,
		SIGNAL( UpdateDetail(const QString&, const QString&, const QString&, const QString&, 
			const QString&, const QString&, const QString&, const QString&,
			const QString&, const QString&, const QString&, const QString&)),

		this,
		SLOT( proxy_UpdateDetail(const QString&, const QString&, const QString&, const QString&, 
			const QString&, const QString&, const QString&, const QString&,
			const QString&, const QString&, const QString&, const QString&))
	);

	connect(transactioninterface, 
		SIGNAL( ErrorCode(const QString&, const QString& )),
		this,
		SLOT( slotError( const QString&, const QString&))
	);

	connect(transactioninterface,
		SIGNAL ( RepoSignatureRequired(const QString&,  const QString&,
			const QString&, const QString&, const QString&,
			const QString&, const QString&, const QString&)),
		this,
		SLOT (proxy_RepoSignatureRequired(const QString&, const QString&,
			const QString&, const QString&, const QString &,
			const QString&, const QString&, const QString &))
	);


	connect(transactioninterface,
		SIGNAL( EulaRequired(const QString&, const QString&, const QString&,
				const QString&)),
		this,
		SLOT( proxy_EulaRequired(const QString&, const QString&, const QString&,
				const QString&))
	);

	connect(transactioninterface,
		SIGNAL (RepoDetail(const QString&, const QString&, bool)),
		this,
		SLOT(proxy_RepoDetail(const QString&, const QString &, bool))
	);

	connect(transactioninterface,
		SIGNAL (RequireRestart(const QString&, const QString&)),
		this,
		SLOT(proxy_RequireRestart(const QString&, const QString &))
	);

	connect(transactioninterface,
		SIGNAL (Message(const QString&, const QString&)),
		this,
		SLOT(proxy_Message(const QString&, const QString &))
	);

	connect(transactioninterface,
		SIGNAL (DistroUpgrade(const QString&, const QString&, const QString &)),
		this,
		SLOT(proxy_DistroUpgrade(const QString&, const QString &, const QString &))
	);

	return transactioninterface;

}



void QPackageKitConnector::sendErrorMessage(QDBusAbstractInterface* interface)
{
    emit ErrorMessage( "DBus error : " 
		+ interface->lastError().name() +  " : " 
		+ interface->lastError().message() );


}


bool QPackageKitConnector::getDetails(QStringList packageId)
{
	QPackageKitTransactionInterface* transactioninterface = connectToInterface();

	if (!transactioninterface)
		return false;

	QDBusReply<void> reply = transactioninterface->GetDetails(packageId);
	if(reply.isValid())
	{
		return true;
	}
	else
	{
		sendErrorMessage(transactioninterface);
		return false;
	}
}



bool QPackageKitConnector::getUpdateDetail(QStringList packageId)
{
	QPackageKitTransactionInterface* transactioninterface = connectToInterface();

	if (!transactioninterface)
		return false;

	QDBusReply<void> reply = transactioninterface->GetUpdateDetail(packageId);
	if(reply.isValid())
	{
		return true;
	}
	else
	{
		sendErrorMessage(transactioninterface);
		return false;
	}
}


bool QPackageKitConnector::acceptEula(QString eulaId)
{
        if(!PackageKit::PolkitClient::instance()->getAuth(AUTH_PACKAGE_EULA_ACCEPT))
        {
            emit ErrorMessage( i18n( "User is not permitted: ") + AUTH_PACKAGE_EULA_ACCEPT );
            return false;
        }

	QPackageKitTransactionInterface* transactioninterface = connectToInterface();

	if (!transactioninterface)
		return false;

	QDBusReply<void> reply = transactioninterface->AcceptEula(eulaId);
	if(reply.isValid())
	{
                kDebug() << "accept eula valid";
		return true;
	}
        else
        {
                kDebug() << "accept eula invalid";
                sendErrorMessage(transactioninterface);
   	        return false;
        }
        return false;
}

bool QPackageKitConnector::getUpdates()
{
	_lastAction    = QString("getUpdates");
	_lastActionArg = false;

	QPackageKitTransactionInterface* transactioninterface = connectToInterface();

	kDebug() << "try to get updates1";
	if (!transactioninterface)
		return false;
	kDebug() << "try to get updates2";
	QDBusReply<void> reply = transactioninterface->GetUpdates("none");
	if(reply.isValid())
	{
		kDebug() << "updates replay valid";
		return true;
	}
	else
	{
		kDebug() << "updates replay not valid";
		sendErrorMessage(transactioninterface);
		return false;
	}
}

bool QPackageKitConnector::getDistroUpgrades()
{
	_lastAction    = QString("getDistroUpgrades");
	_lastActionArg = false;

	QPackageKitTransactionInterface* transactioninterface = connectToInterface();

	if (!transactioninterface)
		return false;
	QDBusReply<void> reply = transactioninterface->GetDistroUpgrades();
	if(reply.isValid())
	{
		kDebug() << "get distro upgrades replay valid";
		return true;
	}
	else
	{
		kDebug() << "get distro upgrades updates replay not valid";
		sendErrorMessage(transactioninterface);
		return false;
	}
}

bool QPackageKitConnector::getDrivers()
{
	_lastAction    = QString("getDrivers");
	_lastActionArg = false;

	QPackageKitTransactionInterface* transactioninterface = connectToInterface();

	if (!transactioninterface)
		return false;

	QDBusReply<void> reply = transactioninterface->WhatProvides("none", "driver", "drivers_for_attached_hardware");

	if(reply.isValid())
	{
		kDebug() << "drivers replay valid";
		return true;
	}
	else
	{
		kDebug() << "drivers replay not valid";
		sendErrorMessage(transactioninterface);
		return false;
	}
}


bool QPackageKitConnector::getRepoList()
{
	QPackageKitTransactionInterface* transactioninterface = connectToInterface();

	if (!transactioninterface)
		return false;
	QDBusReply<void> reply = transactioninterface->GetRepoList("none");
	if(reply.isValid())
	{
		kDebug() << "get repolist valid";
		return true;
	}
	else
	{
		kDebug() << "get repolist not valid";
		sendErrorMessage(transactioninterface);
		return false;
	}
}


bool QPackageKitConnector::refreshCache(bool force)
{
        if(!PackageKit::PolkitClient::instance()->getAuth(AUTH_SYSTEM_SOURCES_REFRESH))
        {
            emit ErrorMessage( i18n( "User is not permitted: ") + AUTH_SYSTEM_SOURCES_REFRESH );
            return false;
        }

	_lastAction    = QString("refreshCache");
	_lastActionArg = force;

	QPackageKitTransactionInterface* transactioninterface = connectToInterface();

	kDebug() << "try to refresh1";
	if (!transactioninterface)
		return false;
	kDebug() << "try to refresh2";
	QDBusReply<void> reply = transactioninterface->RefreshCache(force);
	if(reply.isValid())
	{
		kDebug() << "refresh replay valid";
		return true;
	}
        else
        {
                kDebug() << "refresh replay not valid";
        	sendErrorMessage(transactioninterface);
       	        return false;
        }
        return false;
}


bool QPackageKitConnector::suggestDaemonQuit()
{
	QPackageKitInterface *interface = new QPackageKitInterface( QString::fromAscii(PK_DBUS_SERVICE),
                                                                QString::fromAscii(PK_DBUS_PATH),
                                                                QDBusConnection::systemBus() );

	QDBusReply<void> reply = interface->SuggestDaemonQuit();
	// we can't expect a valig reply, as packagekitd
	// isn't able to answer when it doesn't run anymore :-)

	return true;
	delete interface;
}






QString QPackageKitConnector::getTid()
{
	QPackageKitInterface *interface = new QPackageKitInterface( QString::fromAscii(PK_DBUS_SERVICE),
                                                                QString::fromAscii(PK_DBUS_PATH),
                                                                QDBusConnection::systemBus() );

	kDebug() << "try get a tid.";
	QDBusReply<QString> tid = interface->GetTid();
	if(tid.isValid())
	{
		kDebug() << "tid: " << tid;
		return tid;
	}
	else
	{
		sendErrorMessage(interface);
		return QString();
	}
	
	delete interface;

}

bool QPackageKitConnector::triggerLastActionAgain()
{
	kDebug() << "Starting last action again..."; 

        if (_lastAction == QString("getUpdates"))
		return getUpdates();
        else if (_lastAction == QString("getDrivers"))
		return getDrivers();
        else if (_lastAction    == QString("refreshCache"))
		return refreshCache(_lastActionArg.toBool());
        else if (_lastAction    == QString("installPackages"))
		return installPackages(_lastActionArg.toStringList());
        else if (_lastAction    == QString("updatePackages"))
		return updatePackages(_lastActionArg.toStringList());
        else if (_lastAction    == QString("searchFile"))
		return searchFile(_lastActionArg.toString());
	else 
		return false;
}

bool QPackageKitConnector::installSignature(QString type, QString key_id, QString package_id)
{
        if(!PackageKit::PolkitClient::instance()->getAuth(AUTH_SYSTEM_TRUST_SIGNING_KEY))
        {
            emit ErrorMessage( i18n( "User is not permitted: ") + AUTH_SYSTEM_TRUST_SIGNING_KEY );
            return false;
        }


	QPackageKitTransactionInterface* transactioninterface = connectToInterface();

	if (!transactioninterface)
		return false;

	QDBusReply<void> reply = transactioninterface->InstallSignature(type, key_id, package_id);
	if(reply.isValid())
	{

		kDebug() << "install sig valid";
		return true;
	}
	else
        {
                kDebug() << "install sig not valid";
        	sendErrorMessage(transactioninterface);
        	return false;
        }
        sendErrorMessage(transactioninterface);
        return false;
}

bool QPackageKitConnector::installPackages(QStringList packageIds)
{
        if(!PackageKit::PolkitClient::instance()->getAuth(AUTH_PACKAGE_INSTALL))
        {
            emit ErrorMessage( i18n( "User is not permitted: ") + AUTH_PACKAGE_INSTALL );
            return false;
        }


    _lastAction    = QString("installPackages");
    _lastActionArg = packageIds;

    QPackageKitTransactionInterface* transactioninterface = connectToInterface();

    if (!transactioninterface)
        return false;

    QDBusReply<void> reply = transactioninterface->InstallPackages(packageIds);
    if(reply.isValid())
    {
	kDebug() << "install valid";
        return true;
    }
    else
    {
        kDebug() << "install not valid";
        sendErrorMessage(transactioninterface);
        return false;
    }
    return false;
}

bool QPackageKitConnector::updatePackages(QStringList packageIds)
{
        if(!PackageKit::PolkitClient::instance()->getAuth(AUTH_SYSTEM_UPDATE))
        {
            emit ErrorMessage( i18n( "User is not permitted: ") + AUTH_SYSTEM_UPDATE );
            return false;
        }


	_lastAction    = QString("updatePackages");
	_lastActionArg = packageIds;


	QPackageKitTransactionInterface* transactioninterface = connectToInterface();

	if (!transactioninterface)
		return false;

	QDBusReply<void> reply = transactioninterface->UpdatePackages(packageIds);
	if(reply.isValid())
	{

		kDebug() << "updatepackages valid";
		return true;
	}
        else
        {
                kDebug() << "updatepackages not valid";
        	sendErrorMessage(transactioninterface);
       	        return false;
        }
        return false;
}


bool QPackageKitConnector::searchFile(const QString& search)
{
	_lastAction    = QString("searchFile");
	_lastActionArg = search;


	QPackageKitTransactionInterface* transactioninterface = connectToInterface();

	if (!transactioninterface)
		return false;

	QDBusReply<void> reply = transactioninterface->SearchFile("none", search);
	if(reply.isValid())
	{

		kDebug() << "searchFile valid";
		return true;
	}
	else
	{
		kDebug() << "searchFile not valid";
		sendErrorMessage(transactioninterface);
		return false;
	}
}



void QPackageKitConnector::proxy_Package( const QString &info, const QString &package_id, const QString &summary)
{
	emit Package(info, package_id, summary);
}

void QPackageKitConnector::proxy_DistroUpgrade( const QString &type, const QString &name, const QString &detail)
{
	emit DistroUpgrade(type, name, detail);
}

void QPackageKitConnector::proxy_Finished(const QString &status, uint runtime)
{
	kDebug() << "Finished [status: " << status << "] [runtime: " << runtime << "]";

	QPackageKitTransactionInterface *transactioninterface = qobject_cast<QPackageKitTransactionInterface *>(QObject::sender());
  
	if ( transactioninterface )
	{
		QString param;
		QDBusReply<QString> reply = transactioninterface->GetRole(param);
		if(!reply.isValid())
                {
			kDebug() << "role replay not valid";
//			sendErrorMessage(transactioninterface);
//			return;
                }
                else
                {
			kDebug() << "role replay valid: " << param << " " << reply.value();

			if (	reply.value() == "get-updates" ||  
				reply.value() == "what-provides" ||
				reply.value() == "search-file" )
				emit GetUpdatesFinished();

			if ( reply.value() == "get-repo-list" )
				emit RepoListFinished();

			if (	reply.value() == "update-system" ||
				reply.value() == "install-package" ||
				reply.value() == "install-packages" ||
				reply.value() == "install-file" ||
				reply.value() == "install-files" ||
				reply.value() == "update-package" ||
				reply.value() == "update-packages" )
				emit InstallFinished();

			kDebug() <<  reply.value() ;
                

			disconnectFromInterface(transactioninterface);
			emit Finished(Exit::fromString(status), runtime);
		}
	} 
	else
	{
		kDebug() << "Can't figure out which transaction is finishing";
	}
}

void QPackageKitConnector::proxy_Transaction(const QString &old_tid, const QString &timespec, bool succeeded, const QString &role, uint duration, const QString &data)
{
	Q_UNUSED(timespec);
	Q_UNUSED(succeeded);
	Q_UNUSED(duration);
	Q_UNUSED(data);


	kDebug() << "Transaction: " << old_tid << " " << role;
}


void QPackageKitConnector::proxy_ProgressChanged(uint percentage, uint subpercentage, uint elapsed, uint remaining)
{
	if(percentage == 101)
	{
		emit NoProgressUpdates();
	}
	else
	{
		emit ProgressChanged(percentage, subpercentage, elapsed, remaining);
	}
}

void QPackageKitConnector::proxy_StatusChanged(const QString &status)
{
    kDebug() << "status changed to: " << status;
    emit StatusChanged(status);
}


void QPackageKitConnector::proxy_RepoSignatureRequired(const QString &package_id, const QString &repository_name,
         const QString  &key_url, const QString &key_userid, const QString &key_id,
         const QString &key_fingerprint, const QString &key_timestamp, const QString &type)
{
	kDebug() << "proxy_RepoSignatureRequired";

	emit RepoSignatureRequired(package_id,  repository_name, key_url, key_userid, 
		key_id, key_fingerprint, key_timestamp, type); 
}


void QPackageKitConnector::proxy_Details(const QString &package_id, const QString &licence, const QString &group, const QString &detail,
	const QString &url, qulonglong size) 
{
	emit Details(package_id, licence, group, detail, url, size); 
}

void QPackageKitConnector::proxy_EulaRequired(const QString &eula_id, const QString &package_id, const QString &vendor_name, const QString &license_agreement)
{
	emit EulaRequired(eula_id, package_id, vendor_name, license_agreement);
}	


void QPackageKitConnector::proxy_RepoDetail(const QString &repo_id, const QString &desc, bool enabled)
{
	emit RepoDetail(repo_id, desc, enabled);
}

void QPackageKitConnector::proxy_UpdateDetail(const QString &package_id, const QString &updates, const QString &obsoletes, const QString &vendor_url,
	const QString &bugzilla_url, const QString &cve_url, const QString &restart, const QString &update_text,
	const QString &changelog, const QString &state, const QString &issued, const QString &updated)
{
	emit UpdateDetail(package_id, updates, obsoletes, vendor_url, bugzilla_url, cve_url, restart, update_text,
		changelog, state, issued, updated);
}

void QPackageKitConnector::proxy_RequireRestart(const QString &type, const QString &details)
{
	emit RequireRestart(type, details);
}

void QPackageKitConnector::proxy_Message(const QString &type, const QString &details)
{
	emit Message(type, details);
}

