#include <QPushButton>
#include <QLayout>
#include <QCheckBox>
#include <QLayout>
#include <QLabel>
#include <QTimer>
#include <QTextBrowser>
#include <QShowEvent>
#include <QGridLayout>
#include <QPixmap>
#include <QProcess>
#include <QProgressBar>
#include <KGlobal>
#include <KIconLoader>
#include <KApplication>
#include <KLocale>
#include <KDebug>
#include <KPushButton>
#include <KStandardDirs>

#include "logics.h"
#include "updater.h"
#include "kupdateapplet_settings.h"
#include "constants.h"
#include "simpleview.h"



SimpleView::SimpleView( Logics *l )
			: logics( l )
{
	QWidget * centralWidget = new QWidget( this );
	setCentralWidget( centralWidget );

	QGridLayout * layout = new QGridLayout( centralWidget );
	layout->setSpacing( 5 );
	layout->setMargin( 10 );

	// ------------------- Header --------------------------

	QLabel * logo = new QLabel( this );
	logo->setAlignment(Qt::AlignTop);
	QPixmap * pic = new QPixmap( KIconLoader::global()->loadIcon( PROGRAM_ICON, KIconLoader::Panel, KIconLoader::SizeHuge ) );
	logo->setPixmap( *pic );

	headline = new QLabel( centralWidget );
	headline->
	setText( i18n( "<b>New software for your system is available</b>" ) );
	headline->setAlignment( Qt::AlignTop );


	// ------------------ Boxes ----------------------------

	title = new QLabel( centralWidget );
	title->setText( i18n( "Choose the updates to be installed:" ) );

	progressText = new QLabel( centralWidget );

	textBrowser = new QTextBrowser( centralWidget );

	recommended = new QCheckBox( i18np( "Install %1 recommended update additionally", 
					"Install %1 recommended updates additionally",  0 ), centralWidget );
	recommended->setChecked( true );


	additionalText = new QLabel( centralWidget );
	additionalText-> setText( i18np( "Install %1 recommended update additionally",
					"Install %1 recommended updates additionally",  0  ) );

	norestart = new QCheckBox( i18n( "Don't install updates that require a restart" ), centralWidget );

	bar = new QProgressBar( centralWidget );


	// -------------------- Buttons ------------------------

	details = new KPushButton( i18n( "Details" ), centralWidget );
	install = new KPushButton( KStandardGuiItem::ok(), centralWidget );
	installDrivers = new KPushButton( KStandardGuiItem::ok(), centralWidget );
	ignoreDrivers = new KPushButton( KStandardGuiItem::cancel(), centralWidget );
	ok = new KPushButton( KStandardGuiItem::close(), centralWidget );
	buttonConfigUpdateRepo = new KPushButton( i18n( "Configure now" ), centralWidget );
	buttonManualInstallation   = new KPushButton( i18n( "Start YaST and install updates manually" ), centralWidget );
	install->setText( i18n( "Install" ) );
	installDrivers->setText( i18n( "Install" ) );
	ignoreDrivers->setText( i18n( "Ignore" ) );
	forceinstall = new KPushButton( i18n( "Install anyway" ), centralWidget );
	forcecheck = new KPushButton( i18n( "Check for updates anyway" ), centralWidget );



	// -------------------------- layout -----------------------

	layout->addWidget( logo, 	   		0, 0, 5, 1 );
	layout->addWidget( headline,       		0, 1, 1, 3 );
	layout->addWidget( title, 	   		1, 1, 1, 3 );
	layout->addWidget( progressText,   		2, 1, 1, 3 );
	layout->addWidget( textBrowser,    		3, 1, 1, 3 );
	layout->addWidget( recommended,    		2, 1, 1, 3 );
	layout->addWidget( additionalText, 		3, 1, 1, 3 );
	layout->addWidget( bar, 	   		3, 1, 1, 3 );
	layout->addWidget( norestart, 	   		4, 1, 1, 3);
	layout->addWidget( details, 			6, 0 );
	layout->addWidget( install, 			6, 2 );
	layout->addWidget( installDrivers, 		6, 2 );
	layout->addWidget( ignoreDrivers, 		6, 1 );
	layout->addWidget( buttonConfigUpdateRepo,	6, 2 );
	layout->addWidget( buttonManualInstallation,	6, 1 );
	layout->addWidget( forceinstall, 		6, 1 );
	layout->addWidget( forcecheck, 			6, 1 );
	layout->addWidget( ok, 				6, 3 );

	// -------------------------- connect -----------------------

	connect( details, SIGNAL( clicked() ), this, SLOT( slotDetailsButtonClicked() ) );
	connect( install, SIGNAL( clicked() ), this, SLOT( slotStartInstall() ) );
	connect( installDrivers, SIGNAL( clicked() ), this, SLOT( slotInstallDrivers() ) );
	connect( ignoreDrivers, SIGNAL( clicked() ), this, SLOT( slotIgnoreDrivers() ) );
	connect( ok, SIGNAL( clicked() ), this, SLOT( hide() ) );

	connect( recommended, SIGNAL( clicked() ), this, SLOT( updateSelection() ) );
	connect( norestart, SIGNAL( clicked() ), this, SLOT( updateSelection() ) );
	connect( buttonConfigUpdateRepo, SIGNAL( clicked() ), this, SLOT( slotConfigUpdateRepoClicked() ) );
	connect( buttonManualInstallation, SIGNAL( clicked() ), this, SLOT( slotManualInstallionClicked() ) );
	connect( forceinstall, SIGNAL( clicked() ), this, SLOT( slotStartInstallForce() ) );
	connect( forcecheck, SIGNAL( clicked() ), this, SLOT( slotStartCheckForce() ) );

	slotStateChanged(  );
}


void SimpleView::slotManualInstallionClicked()
{
	hide();
	logics->startManualInstallation();
}

void SimpleView::slotConfigUpdateRepoClicked()
{
	hide();
	logics->startConfigureUpdateRepo();
}



void SimpleView::slotDetailsButtonClicked()
{
	hide();
	emit showDetailView();
}

void SimpleView::slotStartInstall()
{
	hide();
	logics->startInstall();
}

void SimpleView::slotStartInstallForce()
{
	hide();
	logics->startInstall( true ); // force
}

void SimpleView::slotInstallDrivers()
{
	hide();
        logics->installDrivers();
}

void SimpleView::slotIgnoreDrivers()
{
        hide();
        logics->checkForUpdates();
}

void SimpleView::slotStartCheckForce()
{
	hide();
	logics->checkForUpdates( true ); // force
}

void SimpleView::updateSelection()
{
	logics->selectStandardPatches();
	logics->deselectAllPackages();

	if ( recommended->isChecked() )
		logics->selectRecommendedPatches();

	if ( norestart->isChecked() )
		logics->deselectRestartPatches();

	if ( logics->selectedRecommendedPatches() == 0
			 && logics->selectedSecurityPatches() == 0 )
		install->setDisabled( true );
	else
		install->setDisabled( false );
}



bool SimpleView::queryClose()
{
	if ( !kapp->sessionSaving() )
	{
		hide();
		return false;
	}
	
	return true;
}


void SimpleView::slotStateChanged()
{
	// hide everything, let's decide later which gui element we want to show
	title->hide();
	progressText->hide();
	recommended->hide();
	textBrowser->hide();
	additionalText->hide();
	norestart->hide();
	install->hide();
	ok->hide();
	details->hide();
	bar->hide();
	buttonConfigUpdateRepo->hide();
	buttonManualInstallation->hide();
	installDrivers->hide();
	ignoreDrivers->hide();
	forcecheck->hide();
	forceinstall->hide();

	switch ( logics->state() )
	{
		case APPLET_CHECKING:
			headline->setText( i18n( "<b>Checking for updates...</b>" ) );
			progressText->setText( "" );
			progressText->show();
			ok->show();
			bar->show();
		break;

		case APPLET_PROBLEM:
			headline->setText( i18n( "<b>Error:</b>" ) );
			textBrowser->show();
			textBrowser->setHtml( "<qt>" + logics->stateDescription() + "</qt>" );
			textBrowser->setMinimumWidth( 500 );
			ok->show();
			buttonManualInstallation->show();
		break;

		case APPLET_NEW_HARDWARE:      
			headline->setText( i18n( "<b>New hardware has been attached.</b>" ) );
			title->show();
			title->setText( i18n( "Do you want to install drivers needed for the following hardware?<br>\"%1\"").arg (logics->stateDescription()) );
			installDrivers->show();
			ignoreDrivers->show();
		break;

		case APPLET_NO_UPDATES:
			headline->setText( i18n( "<b>No Updates Available</b>" ) );
			ok->show();
		break;

		case APPLET_PKGMGR_UPDATES:
			headline->setText( i18n( "<b>New software for your system is available</b>" ) );
			title->setText( i18n ( "Updates that affect your updater itself are available." ) );
			title->show();
			recommended->setDown( true );
			install->show();
			ok->show();
		break;

		case APPLET_ONLY_OPTIONAL_UPDATES:
		case APPLET_UPDATES:
		case APPLET_CRITICAL_UPDATES:

			updateSelection();

			headline->setText( i18n( "<b>New software for your system is available</b>" ) );
			title->show();
			title->setText( i18np( "%1 security update is available",
										 "%1 security updates are available",
										 logics->securityPatches()  ) );

			if ( logics->recommendedPatches() > 0 )
			{
				if ( logics->updaterCapabilities()
                                     & Updater::CanSelectIndividualUpdates )
				{
					recommended->setText( i18np
							 ( "Install %1 recommended update additionally",
								 "Install %1 recommended updates additionally",
								 logics->recommendedPatches() ) );
					recommended->show();
				}
				else
				{
					additionalText->setText( i18np
							 ( "%1 recommended update is available",
								 "%1 recommended updates are available",
								 logics->recommendedPatches() ) );
					additionalText->show();
				}
		}


		install->show();
		install->setEnabled( true );
		details->show();
		ok->show();

		if ( logics->restartPatches() > 0 )
			norestart->show();

		if ( logics->securityPatches() == 0 && logics->recommendedPatches() == 0 )
		{
			install->setEnabled( false );



			if ( ( logics->updaterCapabilities()  
                               & Updater::CanProvidePackages )  && UpdaterSettings::self()->showPackages() )
			{

				if ( logics->patches() > 0 && logics->packages() == 0 )
				title->setText( i18np
					( "1 optional update is available.<br>You can select optional updates in the detailed dialog.",
					"%1 optional updates are available.<br>You can select optional updates in the detailed dialog.",
					logics->patches()  ) );

				if ( logics->packages() > 0 && logics->patches() == 0 )
				title->setText( i18np
					( "%1 optional upgrade is available.<br>You can select optional upgrades in the detailed dialog.",
					"%1 optional upgrades are available.<br>You can select optional upgrades in the detailed dialog.",
					logics->packages() ) );

				if ( logics->packages() > 0 && logics->patches() > 0 )
				title->setText( i18np( "%1 optional update and upgrade is available.<br>"
					"You can select optional updates and upgrades in the detailed dialog.",
					"%1 optional updates and upgrades are available.<br>"
					"You can select optional updates and upgrades in the detailed dialog.",
					logics->patches() + logics->packages()  ) );
			}
			else
			{
				if ( logics->patches() > 0 )
				title->setText( i18np
					( "1 optional update is available.<br>You can select optional updates in the detailed dialog.",
					"%1 optional updates are available.<br>You can select optional updates in the detailed dialog.",
					logics->patches()  ) );
			}




		}
		break;

		case APPLET_INSTALLING:
			headline->setText( i18n( "<b>Updating...</b>" ) );
			progressText->show();
			progressText->setText( "" );
			ok->show();
			bar->show();
		break;

		case APPLET_NO_UPDATE_REPO:
			headline->setText( i18n ( "<b>Unable to check whether updates are available.</b>" ) );
			title->show();
			title->setText( i18n("Please configure an update repository."));
			ok->show();
			buttonConfigUpdateRepo->show();
		break;

		case APPLET_NO_NETWORK:
			headline->setText( i18n ( "<b>Unable to check whether updates are available.</b>" ) );
			title->show();
			title->setText( i18n( "Please connect to the internet." ) );
			ok->show();
		break;

		case APPLET_SYSTEM_BUSY_WHILE_CHECK:
			headline->setText( i18n( "<b>Unable to check whether updates are available.</b>" ) );
			title->show();
			title-> setText( i18n( "The system is busy. Another check will start soon." ) );
			ok->show();
			forcecheck->show();
		break;

		case APPLET_SYSTEM_IS_ON_BATTERY_WHILE_CHECK:
			headline-> setText( i18n( "<b>Unable to check whether updates are available.</b>" ) );
			title->show();
			title->setText( i18n( "The system is on battery. Please connect a power supply." ) );
			ok->show();
			forcecheck->show();
		break;

		case APPLET_SYSTEM_BUSY_WHILE_INSTALL:
			headline->setText( i18n( "<b>Unable to check whether updates are available.</b>" ) );
			title->show();
			title-> setText( i18n( "The system is busy. Another check will start soon." ) );
			ok->show();
			forceinstall->show();
		break;

		case APPLET_SYSTEM_IS_ON_BATTERY_WHILE_INSTALL:
			headline-> setText( i18n( "<b>Unable to check whether updates are available.</b>" ) );
			title->show();
			title->setText( i18n( "The system is on battery. Please connect a power supply." ) );
			ok->show();
			forceinstall->show();
		break;

	}

	adjustSize();
	updateGeometry();
}


void SimpleView::slotProgressChanged()
{
	bar->setValue( logics->progress() );
	bar->setMaximum( logics->progressTotalSteps() );
	progressText->setText( logics->progressDescription() );
}


void SimpleView::slotSelectionChanged()
{
	// NOP
	
	// do not call updateSelection because that would
	// revert anything changed by the detail window
}


void SimpleView::showEvent(QShowEvent *e)
{
	Q_UNUSED (e);

	// this construct need some explaination:
	// on startup the former applet state is recovered
	// if the user clicks the tray icon the first t

	if ( logics->state() != APPLET_NEW_HARDWARE )
	{
		emit checkAgain();
		disconnect( this, SIGNAL( checkAgain() ), 0, 0 );
	}

	// we need to update the selection because
	// the detailview might have changed the selection
	// in the meantime

	if ( logics->state() == APPLET_ONLY_OPTIONAL_UPDATES ||
 		logics->state() == APPLET_UPDATES || 
                logics->state() == APPLET_CRITICAL_UPDATES )
		updateSelection();
}




#include "simpleview.moc"

// vim: tabstop=2:noet:sw=2
