#include <QLabel>
#include <QMenu>
#include <QMouseEvent>
#include <kdialog.h>
#include <qtimer.h>
#include <qtooltip.h>
#include <qlayout.h>
#include <kdebug.h>
#include <kiconloader.h>
#include <kpassivepopup.h>
#include <klocale.h>
#include <kaction.h>
#include <kapplication.h>

#include "trayicon.h"
#include "updater.h"
#include "kupdateapplet_settings.h"
#include "logics.h"


TrayIcon::TrayIcon( Logics* l )
	: KSystemTrayIcon( 0 )
	, logics( l )
{

	kDebug() << "loading tray..." << endl;

	setIcon( KIconLoader::global()->loadIcon( TRAY_ICON_GREEN, KIconLoader::Panel ) );

	connect(this, SIGNAL( activated( QSystemTrayIcon::ActivationReason ) ),
					this, SLOT( trayActivated( QSystemTrayIcon::ActivationReason ) ) );

	// add menu items to tray icon menu
	_edit_sources_action = new KAction( i18n( "Add/Remove Update Sources..." ), 0 );
	connect( _edit_sources_action, SIGNAL( triggered() ),
						this, SLOT( slotConfigurePlugin() ) );

	_configure_applet_action = new KAction( i18n( "Configure Applet..." ), 0 );
	connect( _configure_applet_action, SIGNAL( triggered() ), 
						this,	SLOT( slotConfigureApplet() ) );

	_check_updates_action = new KAction( i18n( "Check Now" ), 0 );
	connect( _check_updates_action, SIGNAL( triggered() ),
						this,	SLOT( slotCheck() ) );

        _start_distroupgrade_action = new KAction( i18n( "Upgrade Distribution..." ), 0 );
        _start_distroupgrade_action->setVisible(false);
        connect( _start_distroupgrade_action, SIGNAL( triggered() ),
                                                this,   SLOT( slotStartDistroUpgrade() ) );

	QMenu* cm = this->contextMenu();
	cm->addAction( _edit_sources_action );
	cm->addAction( _configure_applet_action );
	cm->addAction( _check_updates_action );
	cm->addAction( _start_distroupgrade_action );
	this->setContextMenu( cm );


	slotStateChanged();
	show();

}

void TrayIcon::slotStartDistroUpgrade()
{
        logics->startDistroUpgrade();
}


void TrayIcon::slotConfigurePlugin()
{
	logics->configurePlugin();
}


void TrayIcon::slotConfigureApplet()
{
	logics->configureApplet();
}


void TrayIcon::slotDistroUpgradeAvailable( bool showItem )
{
        _start_distroupgrade_action->setVisible( showItem );
}


void TrayIcon::slotCheck()
{
	logics->checkForUpdates();
}


TrayIcon::~TrayIcon()
{
}


void TrayIcon::slotStateChanged()
{

	_check_updates_action->setEnabled( true );

	switch ( logics->state() )
	{
		case APPLET_CHECKING:
			_check_updates_action->setEnabled( false );
			setIcon( KIconLoader::global()->loadIcon( TRAY_ICON_CHECKING, KIconLoader::Panel ) );
			setToolTip( i18n( "Checking for updates..." ) );
			break;

		case APPLET_PROBLEM:
			setIcon( KIconLoader::global()->loadIcon( "dialog-warning", KIconLoader::Panel ) );
			setToolTip( "<qt>" + i18n( "Error:\n" ) + logics->stateDescription() + "</qt>" );
			break;

		case APPLET_NO_UPDATES:
			setIcon( KIconLoader::global()->loadIcon( TRAY_ICON_GREEN, KIconLoader::Panel ) );
			setToolTip( i18n( "No Updates Available" ) );
			break;

		case APPLET_UPDATES:
			setToolTip( i18n( "New software updates are available." ) );
			setIcon( KIconLoader::global()->loadIcon( TRAY_ICON_YELLOW, KIconLoader::Panel ) );
			break;

		case APPLET_ONLY_OPTIONAL_UPDATES:
			setToolTip( i18n( "New software updates are available." ) );
			setIcon( KIconLoader::global()->loadIcon( TRAY_ICON_OPTIONAL, KIconLoader::Panel ) );
			break;

		case APPLET_NO_NETWORK:
		case APPLET_SYSTEM_IS_ON_BATTERY_WHILE_CHECK:
		case APPLET_SYSTEM_IS_ON_BATTERY_WHILE_INSTALL:
		case APPLET_SYSTEM_BUSY_WHILE_CHECK:
		case APPLET_SYSTEM_BUSY_WHILE_INSTALL:
			_check_updates_action->setEnabled( false );
			setToolTip( i18n( "Unable to check whether updates are available." ) );
			setIcon( KIconLoader::global()->loadIcon( TRAY_ICON_GRAY, KIconLoader::Panel ) );
			break;

		case APPLET_PKGMGR_UPDATES:
		case APPLET_CRITICAL_UPDATES:
			setToolTip( i18n( "New software updates are available." ) );
			setIcon( KIconLoader::global()->loadIcon( TRAY_ICON_RED, KIconLoader::Panel ) );
			break;

		case APPLET_INSTALLING:
			_check_updates_action->setEnabled( false );
			setIcon( KIconLoader::global()->loadIcon( TRAY_ICON_INSTALLING, KIconLoader::Panel ) );
			setToolTip( i18n( "Updating..." ) );
			break;

		case APPLET_NO_UPDATE_REPO:
			setToolTip( i18n( "Unable to check whether updates are available." ) );
			setIcon( KIconLoader::global()->loadIcon( TRAY_ICON_NO_UPDATE_REPO, KIconLoader::Panel ) );
			break;

		case APPLET_NEW_HARDWARE:
			setToolTip( i18n( "New hardware has been attached." ) );
			setIcon( KIconLoader::global()->loadIcon( TRAY_ICON_HARDWARE, KIconLoader::Panel ) );
		break;

		default:
			setToolTip( QString() );
			break;
	}
}


void TrayIcon::trayActivated( ActivationReason reason )
{
	if ( reason == QSystemTrayIcon::Trigger )
		emit trayClicked();
}


#include "trayicon.moc"
// vim:tabstop=2:noet:sw=2
