#include <kglobal.h>
#include <kconfig.h>
#include <kdebug.h>
#include <kservice.h>
#include <klocale.h>
#include <qtimer.h>
#include <qmessagebox.h>

#include "notification.h"
#include "trayicon.h"
#include "updaterapplication.h"
#include "kupdateapplet_settings.h"
#include "simpleview.h"
#include "updater.h"
#include "mainwindow.h"
#include "constants.h"
#include "logics.h"


UpdaterApplication::UpdaterApplication()
	: KUniqueApplication( true, false )
	, _main   ( 0L )
	, _updater( 0L )
	, _simple ( 0L )
	, _logics ( 0L )
	, _tray	  ( 0L )
	, _notify ( 0L )

{

	_logics = new Logics();
	_simple = new SimpleView( _logics );
	_main   = new MainWindow( _logics );
	_tray	= new TrayIcon( _logics );
	_notify	= new Notification( _logics, 0L );

	connect ( this, 	SIGNAL( pluginError( const QString & ) ),
				 		_logics, SLOT( slotPluginError( const QString & ) ) );
	connect ( _logics,	SIGNAL(configurationChanged() ),
						this,		SLOT( slotConfigChanged() ));

	// connect detailed view
	connect ( _logics, SIGNAL( selectionChanged() ),
						_main,	 SLOT( slotSelectionChanged()));
	connect ( _logics, SIGNAL( stateChanged() ),
						_main,	 SLOT( slotStateChanged() ));
	connect ( _logics, SIGNAL( configurationChanged() ),
						_main,	 SLOT( slotConfigChanged() ));
	connect ( _logics, SIGNAL( resolvableDescription( const QString&, int )),
						_main, SLOT( slotGotDescription( const QString&, int ) )); 


	// connect simplie view
	connect ( _logics, SIGNAL( selectionChanged() ),
						_simple, SLOT( slotSelectionChanged()));
	connect ( _logics, SIGNAL( stateChanged() ),
						_simple, SLOT( slotStateChanged() ));
	connect ( _logics, SIGNAL( progressChanged() ),
						_simple, SLOT( slotProgressChanged() ));

	connect ( _simple, SIGNAL( showDetailView() ),
						_main,	 SLOT( show() ));


	// connect notifications
	connect ( _logics, SIGNAL( stateChanged() ),
						_notify, SLOT( slotStateChanged() ));
	connect ( _logics, SIGNAL( progressChanged() ),
						_notify, SLOT( slotProgressChanged() ));
	connect ( _notify, SIGNAL( showSimpleView() ),
						_simple, SLOT( show() ));
	connect ( _logics, SIGNAL( message(const QString&) ),
						_notify, SLOT( slotMessage(const QString&) ));
        connect ( _logics, SIGNAL( distroUpgradeNotification( const QStringList& )),
                                                _notify, SLOT( slotDistroUpgradeNotification( const QStringList& ) ));

	connect ( _logics, SIGNAL( systemProfile() ),
						_notify, SLOT( slotSystemProfile() ));

	// connect tray icon
	connect ( _tray,	SIGNAL( trayClicked() ) ,
						this,		SLOT( slotTrayClicked() ));
	connect ( _logics,SIGNAL( stateChanged() ),
						_tray,	SLOT( slotStateChanged() ));

        connect ( _logics,      SIGNAL( distroUpgradeAvailable( bool ) ),
                                                _tray,          SLOT( slotDistroUpgradeAvailable( bool ) ));

        // there is no main widget in Qt4, so may be we connect the
        // _main widget object to the quit slot
        //connect( _main, SIGNAL(), SLOT() ); ???
        
	connectToPlugin();


	// set the initial applet state 

	int st = UpdaterSettings::self()->state();
	_logics->setState(st);

	if( st == APPLET_UPDATES ||
			st == APPLET_CRITICAL_UPDATES ||
			st == APPLET_PKGMGR_UPDATES ||
			st == APPLET_ONLY_OPTIONAL_UPDATES )
	{
		// we know there are updates; no need to check again until
		// the user clicks on the tray icon or the next timeout
		connect( _simple, SIGNAL( checkAgain() ), _logics, SLOT( slotAutoCheckForUpdates() ) );
		_logics->addDummyPatch(st);
	}
	else if ( st != APPLET_NO_UPDATES && st != APPLET_NO_UPDATE_REPO )
	{
			// weird state, better check immediately
			// applet->setState( APPLET_PROBLEM );
			QTimer::singleShot( 300, _logics , SLOT( slotAutoCheckForUpdates() ) );
	}

}



void UpdaterApplication::connectToPlugin()
{

	// disconnect old plugin and delete it
	if ( _updater )
	{
		disconnect( _updater, 0, _main, 0 );
		disconnect( _main, 0, _updater, 0 );

		_updater->shutdown();
		delete _updater;
		_updater = NULL;
	}

	// read new plugin from config and connect to it
	KService::Ptr service = selectPlugin( UpdaterSettings::self()->backendPlugin() );

	if ( service )
		loadPlugin( service );
	else
		emit( pluginError( i18n( "Backend plugin is not properly configured.\n"
					"Please use the configuration dialog to choose the correct one." ) ) );

	if ( _updater )
		setupUpdater( _updater );

}


bool UpdaterApplication::loadPlugin( KService::Ptr service )
{
    QString errString;

    Updater* plugin = service->createInstance<Updater>( 0, QVariantList(), &errString );

    if ( plugin )
    {
        _updater = plugin;
        kDebug() << "Backend plugin \"" << service->name() << "\" loaded.";
        return true;
    }

    emit pluginError( errString );

    return false;
}


void UpdaterApplication::slotConfigChanged()
{
    kDebug() << "Config has changed. Reloading backend plugin...";
    connectToPlugin();
}


KService::Ptr UpdaterApplication::selectPlugin( const QString& backendName )
{
    int pluginCount = 0;
    KService::Ptr service;

    kDebug() << "Searching for desired backend plugin " << backendName << " ...";

    KService::List offers = KServiceTypeTrader::self()->query( "kupdateapplet/Plugin" );
    KService::List::ConstIterator iter;

    // ----------------------------------------------------------
    // Try to load selected plugin

    for ( iter = offers.begin(); iter != offers.end(); ++iter )
    {
        service = *iter;
        pluginCount++;

        kDebug() << "Found backend plugin \"" << service->name() << "\"";

        if ( service->name() == backendName )
            return service;
    }

    kDebug() << "Unable to find desired backend plugin!";

    // ----------------------------------------------------------
    // error, if no Plugin found

    if ( pluginCount == 0 )
    {
        QMessageBox::critical( NULL, PROGRAM_DISPLAY_NAME,
                               QString( i18n( "Unable to find any backend plugin. "
		 "Please install one and restart %1", QString( PROGRAM_DISPLAY_NAME ) ) ) );
        return KService::Ptr();
    }


    // ----------------------------------------------------------
    // Try to load default plugin

    if ( backendName != DEFAULT_BACKEND )
    {
        kDebug() << "Trying with default (" << DEFAULT_BACKEND << ") ... ";

        KService::Ptr backend;
        backend = selectPlugin( DEFAULT_BACKEND );

        if ( !backend.isNull() )
        {
            kDebug() << "default backend found. ";

            UpdaterSettings::self()->setBackendPlugin( DEFAULT_BACKEND );
            UpdaterSettings::self()->writeConfig();

            return backend;
        }
    }


    // ----------------------------------------------------------
    // when only one plugin exists select this
    if ( pluginCount == 1 )
        return service;

    // ----------------------------------------------------------
    // when more then one plugin exist show config dialog
    if ( pluginCount > 1 )
			_logics->configureApplet();

    return KService::Ptr();
}

void UpdaterApplication::setupUpdater( Updater *updater )
{

	_logics->setUpdaterCapabilities(updater->capabilities());

	// connect logics => updater
	connect( _logics,  SIGNAL( install() ),
					 _updater, SLOT( slotStartInstall() ) );
	connect( _logics,  SIGNAL( configure() ), 
					 _updater, SLOT( slotConfigureUpdater() ) );
	connect( _logics,  SIGNAL( checkUpdates( QTreeWidget*, QTreeWidget*, QTreeWidget*) ), 
					 _updater, SLOT( slotPopulateLists(QTreeWidget*,  QTreeWidget* , QTreeWidget* )));
	connect( _logics,  SIGNAL( checkDrivers( QTreeWidget*, QTreeWidget*, QTreeWidget* ) ), 
					 _updater, SLOT( slotCheckDrivers(QTreeWidget*, QTreeWidget*, QTreeWidget*)));
	connect( _logics,  SIGNAL( checkFirmware( QTreeWidget*, QTreeWidget*, QTreeWidget* ) ), 
					 _updater, SLOT( slotCheckFirmware(QTreeWidget*, QTreeWidget*, QTreeWidget*)));
	connect( _logics,  SIGNAL( getResolvableDescription( QTreeWidgetItem *, int  ) ),
					 _updater, SLOT( slotResolvableSelected(QTreeWidgetItem* , int) ));
	connect( _logics,  SIGNAL( stopPackageManagementDaemon( ) ),
					 _updater, SLOT( slotStopPackageManagementDaemon( )));
	connect( _logics,  SIGNAL( checkDistroUpgrade( ) ),
					 _updater, SLOT( slotCheckDistroUpgrade( )));




	// connect updater => logics
	connect( _updater, SIGNAL( appletError( const QString& )),
					 _logics,  SLOT( slotAppletError ( const QString& )));
	connect( _updater, SIGNAL( pluginNeedsConfiguration( int  )),
					 _logics,  SLOT( slotPluginNeedsConfiguration( int  )));
	connect( _updater, SIGNAL( checkFinished()),
					 _logics,  SLOT( slotCheckFinished()));
	connect( _updater, SIGNAL( installFinished()),
					 _logics,  SLOT( slotInstallFinished()));
	connect( _updater, SIGNAL( returnDescription (const QString&, int)),
					 _logics,	 SLOT( slotResolvableDescription ( const QString&, int) ));
	connect( _updater, SIGNAL( progress (int,int,const QString&)),
					 _logics,	 SLOT( slotProgress (int,int, const QString&) ));
	connect ( _updater, SIGNAL( message(const QString&) ),
						_logics, SLOT( slotAppletMessage(const QString&) ));
	connect (_updater, SIGNAL ( distroUpgrade( const QString&, const QString&, const QString& ) ),
					_logics, SLOT( slotDistroUpgrade( const QString&, const QString&, const QString& )));

}

UpdaterApplication::~UpdaterApplication()
{

	kDebug() << "~UpdaterApplication()" << endl;
	if ( _updater )
	{
		_updater->shutdown();
		delete _updater;
	}
	delete _main;
	delete _simple;
	delete _notify;
	delete _logics;
}


void UpdaterApplication::slotTrayClicked()
{
	// logic for showing and hinding the view

	if (_main->isVisible() || _simple->isVisible() )
	{
		_main->hide();
		_simple->hide();
		return;
	}
	
	// if no update repo is defined don't show detailed view but simple view 
	if ( _logics->state() == APPLET_NO_UPDATE_REPO ) 
	{
		_simple->show();
		return;
	}

	if ( UpdaterSettings::self()->noSimpleView() )
		_main->show();
	else
		_simple->show();
}

#include "updaterapplication.moc"

// vim: tabstop=2:noet:sw=2
