
#ifndef UPDATER_ERROR_H
#define UPDATER_ERROR_H

#include <kdemacros.h>
#include <QString>

/**
 * Class to pass errors from plugin
 * layers up to the application.
 */
class KDE_EXPORT UpdaterError
{
 public:
  
  enum ErrorType
  {
    ErrorAccessDenied,
    ErrorOther,
    ErrorUnknown
  };
    
  /**
   * construct an unknown error type from
   * a message
   */
  UpdaterError( const QString &message );
  /**
   * destructor
   */
  virtual ~UpdaterError();

  /**
   * construct an error based on the error
   * type, with the default message per
   * type
   */
  UpdaterError( ErrorType errortype );  
  /**
   * construct an error based on a
   * type of error an a context
   * message
   */
  UpdaterError( ErrorType errortype,
               const QString &message );  
  /**
   * set the details of the error
   * this information should be displayed only
   * in a "more.." or "details.." view.
   */
  void setErrorDetails( const QString &details );
  
  /**
   * true if there are detailed information
   */
  bool hasDetails() const;
  
  /**
   * get the detailed information for this error
   */
  QString errorDetails() const;
  
  /**
   * return the error message
   */
  QString errorMessage() const;

 private:
  ErrorType _type;
  QString _details;
  QString _message;
};

#endif
