#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/poll.h>
#include <sys/types.h>
#include <fcntl.h>

#include <qobject.h>
#include <qtimer.h>

#include <kglobal.h>

#include <kgenericfactory.h>
#include <k3process.h>
#include <kdebug.h>
#include <klocale.h>
#include <kconfig.h>
#include <kapplication.h>
#include <kstandarddirs.h>
#include "zyppupdater.h"
#include "zypplistitem.h"
#include "mainwindow.h"	// TODO
#include "licensedialog.h"
#include "../constants.h"

K_EXPORT_COMPONENT_FACTORY( kupdateapplet_zypp, KGenericFactory<ZYppUpdater>( "kupdateapplet_zypp" ) )

ZYppUpdater::ZYppUpdater( QObject *parent, const QStringList& ) 
    : Updater( parent )
    , _process(0L)
    , _state(Unknown)
    , _update_counter(0)
    , _patchList(0)
    , _packageList(0)
    , _found_update_status_tag(false)
    , _error(false)
    , restartUpdater(false)

{

	licenseDialog = new LicenseDialog(NULL);

	// For ease announce that we have been loaded.
	kDebug() << "ZyppUpdater plugin loaded";
	kDebug() << "Initializing zypp backend";

	_xml_reader.setContentHandler(this);
	slotResetXmlStream();
}

Updater::Capabilities ZYppUpdater::capabilities() const
{
    return ( Updater::CanProvidePackages 
             | Updater::CanSelectIndividualUpdates);
}

bool ZYppUpdater::checkRegistrationRan()
{
	QFile file("/var/lib/YaST2/registrationDidRun");
	return file.exists();
}

void ZYppUpdater::slotInstallProcessExited (int exitCode, QProcess::ExitStatus exitStatus)
{
	Q_UNUSED(exitStatus);

	kDebug() << "install process exited.";

	bool  err = handleMessages();

	if (exitCode == 103 )
	{
		kDebug() << "restarting now...";
		system("( sleep 2; kupdateapplet ) & ");
		kapp->quit();
	}

	_install_stderr_buffer.truncate(0);
	_install_stdout_buffer.truncate(0);
	delete _process;
	_process = 0L;


	_silent=true;
	if (!err)
		doCheckForUpdates();
}

void ZYppUpdater::slotResetXmlStream()
{
	// remove if 2 programs are run at the same time
	_xml_source.setData(QString());
	_last_xml_parse_code = true;
	_xml_reader.parse( &_xml_source,
        true /* incremental parsing*/);
}

bool ZYppUpdater::handleMessages()
{
	bool err=false;
	QString message;

	if (!_stderr_buffer.isEmpty())
		_errorList+=_stderr_buffer;

	if (!_warningList.isEmpty())
		for (int i=0; i < _warningList.size(); ++i)
			message +=  _warningList[i] + "<br>";

	if (!_errorList.isEmpty())
	{
		err=true;
		for (int i=0; i < _errorList.size(); ++i)
			message += _errorList[i] +  "<br>";

		appletError(message);
	}
	_warningList.clear();
	_infoList.clear();
	_errorList.clear();
	_stderr_buffer.truncate(0);
	return err;

}

void ZYppUpdater::slotInstSourceProcessExited()
{
	delete _process;
	_process = 0;
}

void ZYppUpdater::slotShowLog()
{
}

void ZYppUpdater::slotHelperFinished( int exitCode, QProcess::ExitStatus exitstatus )
{
	kDebug() << "check process finished... with exitCode" << exitCode << (int) exitstatus;

	if (exitstatus == QProcess::NormalExit) {
		kDebug()  << "ExitStatus: " << exitCode;
		if (exitCode != 0 && exitCode != 100  &&
				exitCode != 101  &&
				exitCode != 102  &&
				exitCode != 103)
				_errorList+=i18n("helper program returned:\n%1", i18n("Exit Status: %1", exitCode));
	}

	if (!_last_xml_parse_code)
		_errorList+=i18n("helper program returned:\n%1").arg("invalid xml");

	delete _process;
	_process = 0L;



	if (handleMessages())	
	{
		// clear the buffer				// some error occured
		_stderr_buffer.truncate(0);
		_buffer.truncate(0);

		emit checkFinished();
		return;
	}

	// clear the buffer
	_stderr_buffer.truncate(0);
	_buffer.truncate(0);
 
	if ( ! _patchList || !_packageList )
	{
		kDebug() << " ! _patchList || ! _packageList ";
		return;
	}

	_patchList->clear();
	_packageList->clear();

	bool pkgmgr_available=false;
	bool critical_available=false;


	kDebug() << "populating... " << _resolvables.count() << " resolvables";
	for ( QList<ZYppResolvable*>::const_iterator it = _resolvables.begin(); it != _resolvables.end(); ++it )
	{
		QTreeWidgetItem *newItem;

		if ( (*it)->resolvableType == RESOLVABLE_PATCH)
			newItem = new ZYppListItem ( (*it), _patchList);
		else
			newItem = new ZYppListItem ( (*it), _packageList );

		newItem->setText(COLUMN_NAME, (*it)->name );
		newItem->setText(COLUMN_SUMMARY, (*it)->summary );
		newItem->setText(COLUMN_TYPE, (*it)->category );
		newItem->setText(COLUMN_NEW_VERSION,(*it)->edition);
		newItem->setText(COLUMN_CATALOG, (*it)->source );
		newItem->setText(COLUMN_RESTART, ((*it)->restart == true ) ? "Yes" : "No" ) ;
		newItem->setText(COLUMN_RESTART_HIDDEN, ((*it)->restart == true) ? "system" : "none" );

		newItem->setFlags(Qt::ItemIsUserCheckable
                	| Qt::ItemIsEnabled
			| Qt::ItemIsSelectable );

		newItem->setCheckState(0, Qt::Unchecked);
		newItem->setSelected(false);

		if ((*it)->category == "security")
			critical_available=true;

		if ((*it)->pkgmanager)
		{
			pkgmgr_available=true;
			newItem->setText(COLUMN_TYPE, "packagemanager" );
		}
	}

	if ( _noUpdateRepositories )
		emit pluginNeedsConfiguration( PLUGIN_CONFIG_NO_UPDATE_REPOS );
	else 
		emit checkFinished();

}

void ZYppUpdater::slotHelperStdout()
{

	// refill the xml data
	_xml_source.setData( _process->readAllStandardOutput() );

	// continue incremental parsing
	_last_xml_parse_code = _xml_reader.parseContinue();

	if ( !_last_xml_parse_code )
	{
		kDebug() << "Incremental parsing error. buffer size: " << _xml_source.data().length();
		kDebug() << errorString();
	}
}

void ZYppUpdater::slotHelperStderr()
{
	_stderr_buffer += _process->readAllStandardError();
}

void ZYppUpdater::shutdown()
{
}


void ZYppUpdater::slotDoRefresh()
{
	_process = new QProcess(this);
	connect(_process, SIGNAL(finished(int, QProcess::ExitStatus)), SLOT(slotRefreshFinished(int, QProcess::ExitStatus)));

	_process->start("/usr/sbin/zypp-refresh-wrapper");
}

void ZYppUpdater::slotRefreshFinished( int exitCode, QProcess::ExitStatus exitStatus)
{
	if ( exitStatus != QProcess::NormalExit || exitCode != 0 )
	{
		QString message = i18n("Refresh failed. Please run 'zypper refresh' as user root.");
//		message += ": ";
//		message += _process->readAllStandardError();
		appletError(message);

		delete _process;
		_process = 0L;
	}
	else
	{
		delete _process;
		_process = 0L;
		slotStartCheckProcess();
	}
}

void ZYppUpdater::doCheckForUpdates()
{
    // NOT NEEDED ANYMORE
	// first of all: check whether registration ran
//	if (!checkRegistrationRan())
//	{
//		kDebug() << "Registration didn't run.";
//		emit pluginNeedsConfiguration( PLUGIN_CONFIG_NO_UPDATE_REPOS );
//		return;
//	}
//	else
//	{
//		kDebug() << "Registration did run.";
//	}


	kDebug() << "Checking for updates...";

	if ( _process )
	{
		kDebug() << "Process still running. Will not run this time.";
		return;
	}

	slotDoRefresh();	// we call doRefresh. If refresh is finished startCheckProcess() is called automatically.

}

void ZYppUpdater::slotStartCheckProcess()
{

	if (_patchList) 
		_patchList->clear();
	if (_packageList) 
		_packageList->clear();

	slotResetXmlStream();
	_error=false;
	_noUpdateRepositories=false;

	_process = new QProcess(this);


	connect(_process, SIGNAL(finished(int, QProcess::ExitStatus)), SLOT(slotHelperFinished(int, QProcess::ExitStatus)));
	connect(_process, SIGNAL(readyReadStandardOutput()), SLOT(slotHelperStdout()));
	connect(_process, SIGNAL(readyReadStandardError()), SLOT(slotHelperStderr()));

	_process->start("/usr/bin/zypper --non-interactive --xmlout list-updates -t package -t patch" );

	kDebug() << "check process started..";
	emit progress( 0 , 1, "");
}


void ZYppUpdater::slotCheckDrivers(QTreeWidget *patchList, QTreeWidget *packageList, QTreeWidget *driverList )
{
	Q_UNUSED(patchList);	
	Q_UNUSED(packageList);	
	Q_UNUSED(driverList);	

	// TODO ask zypper for drivers
	emit checkFinished();
}

void ZYppUpdater::slotCheckFirmware(QTreeWidget *patchList, QTreeWidget *packageList, QTreeWidget *driverList )
{
	Q_UNUSED(patchList);	
	Q_UNUSED(packageList);	
	Q_UNUSED(driverList);	

	// TODO ask zypper for drivers
	emit checkFinished();
}

void ZYppUpdater::slotPopulateLists(QTreeWidget *patchList, QTreeWidget *packageList, QTreeWidget *driverList )
{
	Q_UNUSED(driverList);	

	kDebug() << "got populate UI request...";
	_patchList = patchList;
	_packageList = packageList;
	doCheckForUpdates();
}

void ZYppUpdater::slotResolvableSelected(QTreeWidgetItem *item, int resolvableType)
{
	// retrieve description
	kDebug() << "updating item";
	ZYppListItem *zi = dynamic_cast<ZYppListItem *>(item);
	if ( zi )
	{
		ZYppResolvable *patch = zi->resolvable();
		emit returnDescription(patch->description, resolvableType);
	}
	else
	{
		emit returnDescription("", resolvableType);
	}
}

void ZYppUpdater::slotStartInstall()
{
	_silent=false;

	if ( _process )
	{
		// TODO update still running
		kDebug() << "Process still running. Will not run this time.";
		return;
	}

  
 
	if (!_patchList || !_packageList)
	{
		kDebug() << "!_patchList || !_packageList!!! ";
		kDebug() << "Install is not allowed yet";
		return;
	}

	QStringList installPatchList;
	QStringList installPackageList;

        {
		QTreeWidgetItemIterator it(_patchList);
		while (*it)
		{
			QTreeWidgetItem *item(*it);

			ZYppListItem *zi = dynamic_cast<ZYppListItem *>(item);

			if ( zi 
			    && ( item->checkState(0) == Qt::Checked )
			    && (item->text(COLUMN_NAME) != "dummypatchforappletstate" )
			   )
			{
				ZYppResolvable *patch = zi->resolvable();

				kDebug() <<  patch->name;
				if ( patch->license.trimmed().isEmpty())
					installPatchList += item->text(COLUMN_NAME);
				else
				{
					licenseDialog->setText(i18n("License of %1", patch->name), patch->license);
					if ( licenseDialog->exec() )
						installPatchList += item->text(COLUMN_NAME);
				}
			}
	                ++it;
		}
        }


        {
		QTreeWidgetItemIterator it(_packageList);
		while (*it)
		{
			QTreeWidgetItem *item(*it);
			ZYppListItem *zi = dynamic_cast<ZYppListItem *>(item);
			if ( zi 
			    && ( item->checkState(0) == Qt::Checked )
			    && (item->text(COLUMN_NAME) != "dummypatchforappletstate" )
			   )
			{
				ZYppResolvable *package = zi->resolvable();

				kDebug() <<  package->name;
				if ( package->license.trimmed().isEmpty())
					installPackageList += item->text(COLUMN_NAME);
				else
				{
					licenseDialog->setText(i18n("License of %1", package->name), package->license);
					if ( licenseDialog->exec() )
						installPackageList += item->text(COLUMN_NAME);
				}
			}
	                ++it;
		}
        }


	if ( installPatchList.isEmpty() && installPackageList.isEmpty() )
	{
		kDebug() << "empty";
		appletError(i18n("No patches and no packages marked for installation."));
		return;
	}

	// prevent command injection
	for (int i=0; i < installPatchList.count(); ++i)
		installPatchList[i].replace(QChar('"'), "");
	for (int i=0; i < installPackageList.count(); ++i)
		installPackageList[i].replace(QChar('"'), "");


	slotResetXmlStream();
	_error=false;

	_process = new QProcess(this);

	kDebug() << "patches:" << installPatchList;
	kDebug() << "packages:" << installPackageList;



	QString command="/usr/sbin/zypper-install";

	QStringList args;
	args << "--patch" << installPatchList <<  "--package" << installPackageList;
	connect(_process, SIGNAL(finished(int, QProcess::ExitStatus)), SLOT(slotInstallProcessExited(int, QProcess::ExitStatus)));
	connect(_process, SIGNAL(readyReadStandardOutput()), SLOT(slotHelperStdout()));
	connect(_process, SIGNAL(readyReadStandardError()), SLOT(slotHelperStderr()));

	_process->start( command, args);

	kDebug() << "install process started..";
	emit progress(0, 1, "");

	_patchList->clear();
	_packageList->clear();
}




void ZYppUpdater::slotConfigureUpdater()
{
	kDebug() << "configure updater";

	QProcess *inst_source = new QProcess(this);
	QStringList args;
	args << "-c" << "/sbin/yast2 inst_source";

	connect(inst_source, SIGNAL(finished(int, QProcess::ExitStatus)),
		SLOT(slotInstSourceProcessExited()));
	inst_source->start( KStandardDirs::findExe("kdesu"), args, QIODevice::NotOpen );
}




// QXmlHandler stuff

bool ZYppUpdater::startDocument()
{
  kDebug() << "start document...";
  _found_update_status_tag = false;
  _error = false;
  _message = false;
	qDeleteAll(_resolvables);
  _resolvables.clear();
  return true;
}

bool ZYppUpdater::characters ( const QString & ch )
{
  if ( _state == UpdateDescription )
  {
    _current_resolvable->description += ch;
  }
  if ( _state == UpdateSummary )
  {
    _current_resolvable->summary += ch;
  }
  if ( _state == UpdateLicense )
  {
    _current_resolvable->license += ch;
  }
  if ( _state == Error )
  {
		_errorList += ch;
  }
  if ( _state == PromptDescription)
  {
	kDebug() << "PromptDescrioption" ;
	kDebug() << ch ;
		_errorList += i18n("Operation was too complex: %1").arg( ch );
  }
  if ( _state == Warning )
  {
		_warningList += ch;
  }
  if ( _state == Info )
  {
		_infoList += ch;
  }

  return true;
}

bool ZYppUpdater::startElement( const QString & namespaceURI, const QString & localName, const QString & qName, const QXmlAttributes & atts  )
{
  Q_UNUSED(namespaceURI);
  Q_UNUSED(localName);


  if ( qName == "message")
  {
		if (atts.value("type") == "error")
			_state=Error;
		else if (atts.value("type") == "warning")
			_state=Warning;
		else if (atts.value("type") == "info")
			_state=Info;
	}
	
  if (qName == "prompt")
  {
	kDebug() << "state Prompt";
    _state = Prompt;
  }

  if (qName == "progress")
  {
    _state = Progress;
    emit progress( atts.value("value").toInt(), 100, atts.value("name"));
  }

  if( qName == "appletinfo" )
  {
	kDebug() << "values" << atts.value("status");
	if (atts.value("status") == "no-update-repositories")
		_noUpdateRepositories = true;
  }

  if ( qName == "update" )
  {
    kDebug() << "found patch..." << qName;
    _current_resolvable = new ZYppResolvable();
    _current_resolvable->name = atts.value("name");
    _current_resolvable->category = atts.value("category");
    _current_resolvable->edition = atts.value("edition");
    _current_resolvable->resolvableType=
      ( atts.value("kind") == "patch" ? RESOLVABLE_PATCH : RESOLVABLE_PACKAGE );
    _current_resolvable->restart= ( atts.value("restart") == "true") ? true : false   ;
    _current_resolvable->pkgmanager= ( atts.value("pkgmanager") == "true") ? true : false   ;
    _current_resolvable->interactive= ( atts.value("interactive") == "true") ? true : false   ;

    _state = Update;
    // ignore sources for now
    return true;
  }
  if ( qName == "description" )
  {
    if (  _state == Update )
    {
      _state = UpdateDescription;
      return true;
    }
    if (  _state == Prompt )
    {
      kDebug() << "state PromptDescription";
      _state = PromptDescription;
    return true;
    }
  }
  if ( qName == "summary" )
  {
    if (  _state == Update )
      _state = UpdateSummary;
    return true;
  }
  if ( qName == "license" )
  {
    if (  _state == Update )
      _state = UpdateLicense;
    return true;
  }
  if ( qName == "source" )
  {
    if ( _state == Update )
    {
      _current_resolvable->source = atts.value("alias");
    }
    return true;
  }
  return true;
}

bool ZYppUpdater::endDocument()
{
  kDebug() << "end document";
  return true;
}

bool ZYppUpdater::endElement( const QString &uri , const QString &localname, const QString &qName )
{
  Q_UNUSED(localname);
  Q_UNUSED(uri);

  if ( qName == "update-status" )
  {
    _state = Unknown;
  }
  if ( (qName == "message") )
  {
    _state = Unknown;
  }
  if ( ( qName == "progress"  ))
  {
    _state = Unknown;
  }
  if ( (qName == "errors") )
  {
    _state = Unknown;
  }
  if ( qName == "description" )
  {
	if (_state == UpdateDescription)
	    _state = Update;
	else 
	{
	    _state = Prompt;
		kDebug() << "end of state PromptDescription";
	}
	
    return true;
  }
  if ( qName == "summary" )
  {
    _state = Update;
    return true;
  }
  if ( qName == "prompt" )
  {
    _state = Unknown;
	kDebug() << "end of state Prompt";
    return true;
  }
  if ( qName == "update" )
  {
    _state = Unknown;
    kDebug() << _current_resolvable->name << " appended";
    _resolvables.append(_current_resolvable);

    kDebug() << _current_resolvable->name;

    return true;
  }

  return true;
}

void ZYppUpdater::slotStopPackageManagementDaemon()
{
	//TODO
}

void ZYppUpdater::slotCheckDistroUpgrade()
{
	//TODO
}


#include "zyppupdater.moc"

// vim: ts=2 noet sw=2
